subroutine interpolate_ages (trcl,surface,inparam,outparam)
        
!Read surfaces, let Python do interpolation and read result back in

use trdefinitions
use version
use m_toolbox

implicit none

include 'mpif.h'

type (tracking_cloud) :: trcl
type (input_parameters) :: inparam
type (output_parameters) :: outparam
type (trsheet), dimension(outparam%nstep) :: surface

integer :: i,j,k,l,err,ntracks_l,nvars,stat
integer :: vermajor,verminor,verstat,verrev
integer :: erosiontype

real :: maxdepth,maxtemp,maxpress

double precision :: activation_time
double precision :: velocity_scale, length_scale, density_scale, tempscale, vex

logical :: isostasyout,isobcout,nestout,compactionout,ptracking,calc_hpressure
logical :: erosion, precipitation, fluvial_erosion, diffusion_erosion, clpartfixed

character(4) :: cistep

!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

do l = outparam%first,outparam%last,outparam%step
    ! only do if at least tracks surfaced for this time step
    ! otherwise python interpolation won't work
    ntracks_l = count(trcl%tracks(:,5) == l)
    if (ntracks_l < 4) cycle

    !--------------------------------------------------------------------------|
    !read all information for this time step
    !--------------------------------------------------------------------------|
    call int_to_char (cistep,4,l)
    open (unit=21,file=trim(inparam%dir)//'OUT/time_'//cistep//'.bin', & 
           status='old',form='unformatted')

    !k is the index for the number of output steps taken
    k = (l-outparam%first)/outparam%step + 1

    ! Read version number
    read (21) vermajor,verminor,verstat,verrev
    if (vermajor/=versionmajor .or. verminor/=versionminor .or. verstat/=versionstat) then
        write(*,'(a,i1,a,i1,a,i1,a,i1,a,i1,a,i1)') 'Tracking-postprocessor v', &
            versionmajor,'.',versionminor,'.',versionstat, &
            ' cannot process output from v',vermajor,'.',verminor,'.',verstat
        write(*,'(a)') 'because the output formats are not compatible. To process this output'
        write(*,'(a,i1,a,i1,a,i1,a)') 'please use trpost v', &
            vermajor,'.',verminor,'.',verstat,'.'
      !call stop_run ('Version conflict in reading restart file')
    endif

    ! Read optional output flags
    read (21) isostasyout,isobcout,nestout,compactionout,ptracking,calc_hpressure,erosion, clpartfixed

    ! read scaling information
    read (21) velocity_scale, length_scale, density_scale, tempscale, vex

    read (21) 

    read (21)
    read (21)
    read (21)
    read (21)
    read (21)
    read (21)
    read (21)
    read (21)
    read (21)
    read (21)
    read (21)
    read (21)

    if (isostasyout) then
        read (21)
        read (21)
        read (21)
    end if
    if (compactionout) read (21)

    read (21)
    read (21)
    read (21)
    if (calc_hpressure) read (21) 
    read (21)
    read (21)
    read (21)
    read (21)
    read (21)
    read (21)
    read (21)
    read (21)
    read (21)

    if (compactionout) read (21)

    read (21)
    read (21)

    read (21)
    read (21)
    read (21)
    read (21)
    read (21)

    read (21)

    ! read surface information
    ! read only first surface
    read (21) surface(k)%nsurface,activation_time,surface(k)%nt

    !calc number of variables
    nvars = outparam%nages + outparam%origin*3 +  &
            outparam%maxdepth + outparam%maxtemp + outparam%maxpressure
    !allocate arrays for this surface
    allocate (surface(k)%x(surface(k)%nsurface),stat=err)
    allocate (surface(k)%y(surface(k)%nsurface),stat=err)
    allocate (surface(k)%z(surface(k)%nsurface),stat=err)
    allocate (surface(k)%icon(3,surface(k)%nt),stat=err)
    allocate (surface(k)%ages(surface(k)%nsurface,nvars),stat=err)
    if (err /= 0) then
       write (*,*) 'Error acllocating surface'
       stop
    end if
    read (21) surface(k)%x
    read (21) surface(k)%y
    read (21) surface(k)%z
    read (21) 
    read (21)
    read (21)
    read (21)
    read (21)
    read (21)
    if (erosion) then
       read (21) !surface(is)%erosion
       read (21) erosiontype, precipitation, fluvial_erosion, diffusion_erosion
       if (precipitation) read (21) !surface(is)%precipitation
       if (fluvial_erosion) read (21) !surface(is)%fluvial_erosion
       if (diffusion_erosion) read (21) !surface(is)%diffusion_erosion
       if (erosiontype == FASTSCAPE_EROSION) read (21) !surface(is)%catchment
    endif
    read (21) surface(k)%icon
    read (21) !surface(k)%convexhull

    close (21)

    !--------------------------------------------------------------------------|
    !write surface and age information for python skript
    !--------------------------------------------------------------------------|
    
    open (unit=22, file='ages.dat', status='replace')
    !write xy position of track and all ages that are calculated
    do i=1,trcl%ntracks
        if (trcl%tracks(i,5)==l) then
            !if track surfaces at this timestep: write last point of track (at surface)
            write(22,'(2ES14.5E2)',advance='no') trcl%x(trcl%tracks(i,1),trcl%tracks(i,3)), &
                trcl%y(trcl%tracks(i,1),trcl%tracks(i,3))
            if (outparam%Mad_AHe_std ==1) write(22,'(ES14.5E2)',advance='no') trcl%Mad_AHe_std(i)
            if (outparam%Mad_Ahe_small ==1) write(22,'(ES14.5E2)',advance='no') trcl%Mad_Ahe_small(i)
            if (outparam%Mad_Ahe_medium ==1) write(22,'(ES14.5E2)',advance='no') trcl%Mad_Ahe_medium(i)
            if (outparam%Mad_Ahe_large ==1) write(22,'(ES14.5E2)',advance='no') trcl%Mad_Ahe_large(i)
            if (outparam%Mad_Ahe_lowdam ==1) write(22,'(ES14.5E2)',advance='no') trcl%Mad_Ahe_lowdam(i)
            if (outparam%Mad_Ahe_moddam ==1) write(22,'(ES14.5E2)',advance='no') trcl%Mad_Ahe_moddam(i)
            if (outparam%Mad_Ahe_hidam ==1) write(22,'(ES14.5E2)',advance='no') trcl%Mad_Ahe_hidam(i)
            if (outparam%Mad_ZHe ==1) write(22,'(ES14.5E2)',advance='no') trcl%Mad_ZHe(i)
            if (outparam%Mad_ArKFs ==1) write(22,'(ES14.5E2)',advance='no') trcl%Mad_ArKFs(i)
            if (outparam%Mad_ArBio ==1) write(22,'(ES14.5E2)',advance='no') trcl%Mad_ArBio(i)
            if (outparam%Mad_ArHorn ==1) write(22,'(ES14.5E2)',advance='no') trcl%Mad_ArHorn(i)
            if (outparam%Mad_ArMusc ==1) write(22,'(ES14.5E2)',advance='no') trcl%Mad_ArMusc(i)
            if (outparam%Mad_Ti_UTh_He ==1) write(22,'(ES14.5E2)',advance='no') trcl%Mad_Ti_UTh_He(i)
            if (outparam%Mad_AFT ==1) write(22,'(ES14.5E2)',advance='no') trcl%Mad_AFT(i)
            if (outparam%Mad_ZFT ==1) write(22,'(ES14.5E2)',advance='no') trcl%Mad_ZFT(i)
            if (outparam%Muscovite ==1) write(22,'(ES14.5E2)',advance='no') trcl%Muscovite(i)
            if (outparam%ZFT ==1) write(22,'(ES14.5E2)',advance='no') trcl%ZFT(i)
            if (outparam%Apatite_U_Th_Pb ==1) write(22,'(ES14.5E2)',advance='no') trcl%Apatite_U_Th_Pb(i)
            if (outparam%Biotite ==1) write(22,'(ES14.5E2)',advance='no') trcl%Biotite(i)
            if (outparam%Rutile_U_Pb ==1) write(22,'(ES14.5E2)',advance='no') trcl%Rutile_U_Pb(i)
            if (outparam%Titanite_U_Pb ==1) write(22,'(ES14.5E2)',advance='no') trcl%Titanite_U_Pb(i)
            if (outparam%Zircon_U_Pb ==1) write(22,'(ES14.5E2)',advance='no') trcl%Zircon_U_Pb(i)
            if (outparam%origin ==1)  then 
                write(22,'(ES14.5E2)',advance='no') trcl%x(trcl%tracks(i,1),trcl%tracks(i,2))
                write(22,'(ES14.5E2)',advance='no') trcl%y(trcl%tracks(i,1),trcl%tracks(i,2))
                write(22,'(ES14.5E2)',advance='no') trcl%z(trcl%tracks(i,1),trcl%tracks(i,2))
            end if
            if (outparam%maxdepth==1) then
                !Get minimum Z value of track
                maxdepth = minval(trcl%z(trcl%tracks(i,1), trcl%tracks(i,2):trcl%tracks(i,3)))
                write(22,'(ES14.5E2)',advance='no') maxdepth
            end if
            if (outparam%maxtemp==1) then
                maxtemp = maxval(trcl%temp(trcl%tracks(i,1), trcl%tracks(i,2):trcl%tracks(i,3)))
                write(22,'(ES14.5E2)',advance='no') maxtemp
            end if
            if (outparam%maxpressure==1) then
                maxpress = abs(minval(trcl%press(trcl%tracks(i,1), trcl%tracks(i,2):trcl%tracks(i,3))))
                write(22,'(ES14.5E2)',advance='no') maxpress
            end if
            write (22, '(a)') ''
        end if
    end do
    close (22)

    
    if (outparam%scaled==1) then
        surface(k)%x = surface(k)%x*length_scale
        surface(k)%y = surface(k)%y*length_scale
        surface(k)%z = surface(k)%z*length_scale
    end if

    open (unit=23, file='surface.dat', status='replace')
    do i=1,surface(k)%nsurface
        write (23, '(2F14.6)') surface(k)%x(i), surface(k)%y(i)
    end do
    close (23)

    !--------------------------------------------------------------------------|
    !call python skript for age interpolation
    !--------------------------------------------------------------------------|

    call system('./interpolate_ages.py')

    !--------------------------------------------------------------------------|
    !read skript output
    !--------------------------------------------------------------------------|

    open (unit=24, file='interpages.dat', status='old')
    !read all interpolated ages from Python output
    do i = 1,surface(k)%nsurface
        read (24,*) surface(k)%ages(i,:)
    end do

    close (24)

end do

open (29,file='ages.dat',iostat=stat,status='old')
if (stat.eq.0) close(29, status='delete')
open (29,file='surface.dat',iostat=stat,status='old')
if (stat.eq.0) close(29, status='delete')
open (29,file='interpages.dat',iostat=stat,status='old')
if (stat.eq.0) close(29, status='delete')

end subroutine interpolate_ages
