#! /usr/bin/env python
#-*- coding: UTF8 -*-

import numpy as np
from scipy.interpolate import griddata
from scipy import signal

def gauss_kern(size, sizey=None):
    """ Returns a normalized 2D gauss kernel array for convolutions """
    size = int(size)
    if not sizey:
        sizey = size
    else:
        sizey = int(sizey)
    x, y = np.mgrid[-size:size+1, -sizey:sizey+1]
    g = np.exp(-(x**2/float(size) + y**2/float(sizey)))
    return g / g.sum()

def blur_image(im, n, ny=None) :
    """ blurs the image by convolving with a gaussian kernel of typical
        size n. The optional keyword argument ny allows for a different
        size in the y direction.
    """
    g = gauss_kern(n, sizey=ny)
    improc = signal.convolve(im, g, mode='valid')
    return(improc)

#-------------------------------------------------------------------------------
# Read input
#-------------------------------------------------------------------------------

print 'Starting Python Interpolation'

#Read pad_age from inputs_of_outputs.txt
with open('input_tracking_outputs.txt') as inparams:
    for i,line in enumerate(inparams):
        if i == 13:
            nbins = float(line.split(' ')[0])
        if i == 17:
            pad_age = float(line.split(' ')[0])
            break

#read surface coordinates
with open('surface.dat') as surfdata:
    surf = [] 
    for line in surfdata:
        surf.append( [float(num) for num in line.split()] )
    surf = np.array(surf)

#read Age data
with open('ages.dat') as agedata:
    ages = []  
    for line in agedata:
        ages.append( [float(num) for num in line.split()] )
    ages = np.array(ages)
    (ntracks, nages) = ages.shape 

#-------------------------------------------------------------------------------
# Do interpolation
#-------------------------------------------------------------------------------

(nsurf, dummy) = surf.shape

interpages = -1.0 * np.ones((nsurf,nages-2))

if ntracks > 3:
    if nbins > 1:
        #smooth input ages
        smoothages = []
        maxx = max(surf[:,0])
        maxy = max(surf[:,1])
        aggregator = [ [[]  for i in xrange(nbins) ] for i in xrange(nbins)] 

        for i, point in enumerate(ages):
            ix = int(point[0]/maxx*nbins)
            iy = int(point[1]/maxy*nbins)
            aggregator[ix][iy].append(i)

        for line in aggregator:
            for area in line:
                if len(area) > 0:
                    collectages = []
                    for i in area:
                        collectages.append(ages[i])
                    collectages = np.array(collectages)
                    smoothages.append(np.mean(collectages,axis=0))

        smoothages = np.array(smoothages)
    else:
        smoothages = ages


    for i in xrange(2,nages):
        interpages[:,i-2] = griddata(smoothages[:,0:2], smoothages[:,i], surf, method='linear', fill_value=pad_age) 

np.savetxt('interpages.dat', interpages, fmt='%14.6e')    

