module m_create_surfaces
contains
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         |
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              CREATE_SURF    May. 2008                                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine create_surf(surface,is,params)

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! for a given surfaces passed as argument, this routine computes
! the 8 fields x,y,z,xn,yn,zn,r,s for each point of the surface
! according to its type and levelt, and the triangulation
!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

use constants
use definitions
!use mpi
use m_toolbox
use m_compute_normals

implicit none

include 'mpif.h'

type (parameters) :: params
type (sheet), dimension(params%ns) :: surface
integer is

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|

integer :: nproc,iproc,ierr,i,j,seed,indix
integer :: err,ntmax,nhmax,npmax,nmax,nmode,nvmax,nnpnmax,nh,nohalt_hull,loc
integer :: dmode,nt,nhull,icircles,irect
integer,dimension(:),  allocatable :: hulltriangles,vis_tlist,vis_elist
integer,dimension(:),  allocatable :: add_tlist
integer,dimension(:,:),allocatable :: vertices,neighbour
logical*1,dimension(:),allocatable :: lt_work,ln_work
double precision :: delta,eps,epsil,delta_angle,delta_radius,angle
double precision :: xx,yy,xini,xend,yini,yend,dx,dy,dist
double precision,dimension(:),  allocatable :: field
double precision,dimension(:,:),allocatable :: points,centres
logical clockwise
character ch
character*72  :: shift
integer nnn(1),nnlist(1),ntrilist(1)
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)

shift=' '
eps = 1.d-10

if (surface(is)%surface_type.lt.0) then
   !---------------------------------
   ! reading surface from ascii file
   !---------------------------------
   if (iproc.eq.0) then
      write (ch,'(i1)') -surface(is)%surface_type
      write(*,*) shift//'surface-'//ch//'.txt'
      open(71,file='surface-'//ch//'.txt',status='old')
      read(71,*) surface(is)%nsurface,surface(is)%nt
      write(*,*) shift//'-> nb of points   :',surface(is)%nsurface
      write(*,*) shift//'-> nb of triangles:',surface(is)%nt
   end if
   call mpi_bcast (surface(is)%nsurface,1,mpi_integer,0,mpi_comm_world,ierr)
   call mpi_bcast (surface(is)%nt,1,mpi_integer,0,mpi_comm_world,ierr)
   deallocate (surface(is)%x,surface(is)%y,surface(is)%z)
   deallocate (surface(is)%xn,surface(is)%yn,surface(is)%zn)
   deallocate (surface(is)%icon)
   allocate (surface(is)%x(surface(is)%nsurface))
   allocate (surface(is)%y(surface(is)%nsurface))
   allocate (surface(is)%z(surface(is)%nsurface))
   allocate (surface(is)%xn(surface(is)%nsurface))
   allocate (surface(is)%yn(surface(is)%nsurface))
   allocate (surface(is)%zn(surface(is)%nsurface))
   allocate (surface(is)%icon(3,surface(is)%nt))
   if (iproc.eq.0) then
      write(*,*) surface(is)%nsurface,surface(is)%nt
      do i=1,surface(is)%nsurface
         read (71,*) surface(is)%x(i),surface(is)%y(i),surface(is)%z(i), &
                     surface(is)%xn(i),surface(is)%yn(i),surface(is)%zn(i)
         if (surface(is)%x(i)<0 .or. surface(is)%x(i)>1 .or. &
             surface(is)%y(i)<0 .or. surface(is)%y(i)>1 .or. &
             surface(is)%z(i)<0 .or. surface(is)%z(i)>1 &
             .and. iproc.eq.0)  then
             write(*,*) 'WARNING: surface node', i, ' for surface', is, ' outside domain!'
             write(*,*) surface(is)%x(i),surface(is)%y(i),surface(is)%z(i)
         end if
         !Set Convex hull from coordinates
         surface(is)%convexhull(i) = 0
         if (surface(is)%y(i) < eps) then
             surface(is)%convexhull(i) = surface(is)%convexhull(i) + 2
         end if
         if (surface(is)%x(i) > 1.d0 - eps) then
             surface(is)%convexhull(i) = surface(is)%convexhull(i) + 4
         end if
         if (surface(is)%y(i) > 1.d0 - eps) then
             surface(is)%convexhull(i) = surface(is)%convexhull(i) + 8
         end if
         if (surface(is)%x(i) < eps) then
             surface(is)%convexhull(i) = surface(is)%convexhull(i) + 16
         end if
         !mark corners
         if (surface(is)%convexhull(i) > 0 .AND. &
             mod(surface(is)%convexhull(i), 6) == 0) then
            surface(is)%convexhull(i) = surface(is)%convexhull(i) + 1
         end if
      enddo
      do i=1,surface(is)%nt
         read (71,*) (surface(is)%icon(j,i),j=1,3)
      enddo
      close (71)
   endif

   call mpi_bcast(surface(is)%x,surface(is)%nsurface,mpi_double_precision,0,mpi_comm_world,ierr)
   call mpi_bcast(surface(is)%y,surface(is)%nsurface,mpi_double_precision,0,mpi_comm_world,ierr)
   call mpi_bcast(surface(is)%z,surface(is)%nsurface,mpi_double_precision,0,mpi_comm_world,ierr)
   call mpi_bcast(surface(is)%xn,surface(is)%nsurface,mpi_double_precision,0,mpi_comm_world,ierr)
   call mpi_bcast(surface(is)%yn,surface(is)%nsurface,mpi_double_precision,0,mpi_comm_world,ierr)
   call mpi_bcast(surface(is)%zn,surface(is)%nsurface,mpi_double_precision,0,mpi_comm_world,ierr)
   call mpi_bcast(surface(is)%convexhull,surface(is)%nsurface,mpi_integer,0,mpi_comm_world,ierr)
   call mpi_bcast(surface(is)%icon,surface(is)%nt*3,mpi_integer,0,mpi_comm_world,ierr)
else
   !----------------
   ! create surface
   !----------------

   seed=1234567

   !delta=2.d0**(-levelt) ! spacing between points on hull

   delta=1.d0 / (2.d0**surface(is)%levelt +1) ! opla

   epsil=(surface(is)%stretch-1)/2.d0

   if (iproc.eq.0) write(8,*) 'create_surfaces: delta=',delta
   if (iproc.eq.0) write(8,*) 'create_surfaces: epsilon=',epsil

   !-----building the surface-------------
   ! type 1  : flat surface
   ! type 2  : rectangular emboss
   ! type 3  : convex spherical emboss
   ! type 4  : concave spherical emboss
   ! type 5  : double rectangular emboss
   ! type 6  : sinus
   ! type 7  : noisy surface
   ! type 8  : double sinus
   ! type 9  : cosinus sinus
   ! type 10 : slope
   ! type 11 : 2D hill
   ! type 12 : rectangular emboss with specified slope
   ! type 13 : slope with 2 kinks
   ! type 14 : slope with 2 kinks, one rounded

   !Introducing a n * n ordered surface for Fastscape
   !Currently as a special case, this might be used as standard for all surfaces
   !later on.

   nodepattern: if (surface(is)%fastscape .OR. surface(is)%rectgrid) then
       ! currently all surfaces are initalized with 2**levelt+2 x 2**levelt+2 nodes
       ! on the convex hull. The reason for this is not entirely clear to me, but I'll
       ! go with it for now --- Matthias Nettesheim, 2016-06-22
       indix = 1
       do i=1,2**surface(is)%levelt +2
           do j=1,2**surface(is)%levelt +2
               surface(is)%x(indix)=dble(j-1)*delta
               surface(is)%y(indix)=dble(i-1)*delta
               indix = indix + 1
           end do
       end do

   else nodepattern
   select case(surface(is)%surface_type)

      case (1,2,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19)
           !if (rand) then
              indix=1
              ! convex hull
              do i=1,2**surface(is)%levelt   +1
                 surface(is)%x(indix)=dble(i-1)*delta
                 surface(is)%y(indix)=0.d0
                 indix=indix+1
              end do
              do i=1,2**surface(is)%levelt   +1
                 surface(is)%x(indix)=1.d0
                 surface(is)%y(indix)=dble(i-1)*delta
                 indix=indix+1
              end do
              do i=1,2**surface(is)%levelt    +1
                 surface(is)%x(indix)=1.d0-dble(i-1)*delta
                 surface(is)%y(indix)=1.d0
                 indix=indix+1
              end do
              do i=1,2**surface(is)%levelt   +1
                 surface(is)%x(indix)=0.d0
                 surface(is)%y(indix)=1.d0-dble(i-1)*delta
                 indix=indix+1
              end do
              !interior nodes
              if(surface(is)%rand) then
                 call random_number (surface(is)%x(indix:surface(is)%nsurface))
                 call random_number (surface(is)%y(indix:surface(is)%nsurface))
              else
                 surface(is)%x(indix:surface(is)%nsurface)=0.5d0
                 surface(is)%y(indix:surface(is)%nsurface)=0.5d0
              end if
              do j=2,2**surface(is)%levelt +1
                 do i=2,2**surface(is)%levelt  +1
                    surface(is)%x(indix)=(((surface(is)%x(indix)-0.5d0)*2.d0)*epsil+dble(i-1))*delta
                    surface(is)%y(indix)=(((surface(is)%y(indix)-0.5d0)*2.d0)*epsil+dble(j-1))*delta
                    indix=indix+1
                 enddo
              enddo
           !else
              !! regular grid of points
              !indix=1
              !do j=1,2**levelt+1
                 !do i=1,2**levelt+1
                    !x(indix)=float(i-1)*delta
                    !y(indix)=float(j-1)*delta
                    !indix=indix+1
                 !end do
              !end do
           !end if

      case (22)
           if (surface(is)%rand) then
              indix=1
              ! convex hull
              do i=1,2**surface(is)%levelt
                 surface(is)%x(indix)=(i-1)*delta
                 surface(is)%y(indix)=0.d0
                 indix=indix+1
              end do
              do i=1,2**surface(is)%levelt
                 surface(is)%x(indix)=1.d0
                 surface(is)%y(indix)=(i-1)*delta
                 indix=indix+1
              end do
              do i=1,2**surface(is)%levelt
                 surface(is)%x(indix)=1.d0-(i-1)*delta
                 surface(is)%y(indix)=1.d0
                 indix=indix+1
              end do
              do i=1,2**surface(is)%levelt
                 surface(is)%x(indix)=0.d0
                 surface(is)%y(indix)=1.d0-(i-1)*delta
                 indix=indix+1
              end do
              if (iproc.eq.0) write(8,*) 'nhull=',indix-1,'for surface ',is
              ! placing rows of points outside, on and outside the rectangle
              eps=0.001
              do irect=-5,3,2
                 xini=surface(is)%sp02+irect*eps
                 xend=surface(is)%sp03-irect*eps
                 yini=surface(is)%sp04+irect*eps
                 yend=surface(is)%sp05-irect*eps
                 dx=(xend-xini)/2**surface(is)%levelt
                 dy=(yend-yini)/2**surface(is)%levelt
                 do i=1,2**surface(is)%levelt
                    xx=xini+(i-1)*dx
                    yy=yini
                    if (xx.ge.0 .and. xx.le.1 .and. yy.ge.0 .and. yy.le.1) then
                       surface(is)%x(indix)=xx
                       surface(is)%y(indix)=yy
                       indix=indix+1
                    end if
                 end do
                 do i=1,2**surface(is)%levelt
                    xx=xend
                    yy=yini+(i-1)*dy
                    if (xx.ge.0 .and. xx.le.1 .and. yy.ge.0 .and. yy.le.1) then
                       surface(is)%x(indix)=xx
                       surface(is)%y(indix)=yy
                       indix=indix+1
                    end if
                 end do
                 do i=1,2**surface(is)%levelt
                    xx=xend-(i-1)*dx
                    yy=yend
                    if (xx.ge.0 .and. xx.le.1 .and. yy.ge.0 .and. yy.le.1) then
                       surface(is)%x(indix)=xx
                       surface(is)%y(indix)=yy
                       indix=indix+1
                    end if
                 end do
                 do i=1,2**surface(is)%levelt
                    xx=xini
                    yy=yend-(i-1)*dy
                    if (xx.ge.0 .and. xx.le.1 .and. yy.ge.0 .and. yy.le.1) then
                       surface(is)%x(indix)=xx
                       surface(is)%y(indix)=yy
                       indix=indix+1
                    end if
                 end do
              end do
              ! throwing remaining points at random
              do while (indix.le.surface(is)%nsurface)
                 xx=ran(seed)
                 yy=ran(seed)
                 if ((xx<surface(is)%sp02-10*eps .or. xx>surface(is)%sp03+10*eps .or. &
                     yy<surface(is)%sp04-10*eps .or. yy>surface(is)%sp05+10*eps) .or. &
                     (xx>surface(is)%sp02+10*eps .and. xx<surface(is)%sp03-10*eps .and. &
                      yy>surface(is)%sp04+10*eps .and. yy<surface(is)%sp05-10*eps)) then
                    surface(is)%x(indix)=xx
                    surface(is)%y(indix)=yy
                    indix=indix+1
                 end if
              end do
           else
              ! regular grid of points
              indix=1
              do j=1,2**surface(is)%levelt+1
                 do i=1,2**surface(is)%levelt+1
                 surface(is)%x(indix)=float(i-1)*delta
                 surface(is)%y(indix)=float(j-1)*delta
                 ! identify convex hull?
                 indix=indix+1
                 end do
             end do
           end if

      case (3,4)
           if (surface(is)%rand) then
              ! convex hull
              indix=1
              do i=1,2**surface(is)%levelt
                 surface(is)%x(indix)=(i-1)*delta
                 surface(is)%y(indix)=0.d0
                 indix=indix+1
              end do
              do i=1,2**surface(is)%levelt
                 surface(is)%x(indix)=1.d0
                 surface(is)%y(indix)=(i-1)*delta
                 indix=indix+1
              end do
              do i=1,2**surface(is)%levelt
                 surface(is)%x(indix)=1.d0-(i-1)*delta
                 surface(is)%y(indix)=1.d0
                 indix=indix+1
              end do
              do i=1,2**surface(is)%levelt
                 surface(is)%x(indix)=0.d0
                 surface(is)%y(indix)=1.d0-(i-1)*delta
                 indix=indix+1
              end do
              nhull=indix-1
              if (iproc.eq.0) write(8,*) 'nhull=',nhull,'on surface ',is
              ! placing rows of nhull points outside, in, and inside the circle
              delta_angle=2.d0*pi/nhull
              delta_radius=0.0025d0*surface(is)%sp04
              do icircles=-4,2
                 do i=0,nhull-1
                    angle=i*delta_angle
                    surface(is)%x(indix)=surface(is)%sp02 + &
                                         (surface(is)%sp04+icircles*delta_radius)*cos(angle)
                    surface(is)%y(indix)=surface(is)%sp03 + &
                                         (surface(is)%sp04+icircles*delta_radius)*sin(angle)
                    indix=indix+1
                 end do
              end do
              ! throwing the rest of the points at random provided they are not
              ! too close to the concentric circles so that it does not disturb
              ! the regular triangulation
              do while (indix.le.surface(is)%nsurface)
                 xx=ran(seed)
                 yy=ran(seed)
                 dist=sqrt((xx-surface(is)%sp02)**2+(yy-surface(is)%sp03)**2)
                 if (dist>(surface(is)%sp04+10*delta_radius) .or. &
                     dist<(surface(is)%sp04-10*delta_radius)) then
                    surface(is)%x(indix)=xx
                    surface(is)%y(indix)=yy
                    indix=indix+1
                 end if
              end do
           else
              ! regular grid of points
              indix=1
              do j=1,2**surface(is)%levelt+1
                 do i=1,2**surface(is)%levelt+1
                    surface(is)%x(indix)=float(i-1)*delta
                    surface(is)%y(indix)=float(j-1)*delta
                    indix=indix+1
                 end do
              end do
           end if

      case default
           call stop_run ('surface type not defined')
   end select
   end if nodepattern

   ! Set Convex hull from coordinates
   ! Convex hull flagging of the points is:
   !              8 (y=1)
   !        25.-------.13
   !          |       |
   ! 16 (x=0) |       | 4 (x=1)
   !          |       |
   !        19.-------.7
   !              2 (y=0)
   !
   do i=1,surface(is)%nsurface
      surface(is)%convexhull(i) = 0
      if (surface(is)%y(i) < eps) then
          surface(is)%convexhull(i) = surface(is)%convexhull(i) + 2
      end if
      if (surface(is)%x(i) > 1.d0 - eps) then
          surface(is)%convexhull(i) = surface(is)%convexhull(i) + 4
      end if
      if (surface(is)%y(i) > 1.d0 - eps) then
          surface(is)%convexhull(i) = surface(is)%convexhull(i) + 8
      end if
      if (surface(is)%x(i) < eps) then
          surface(is)%convexhull(i) = surface(is)%convexhull(i) + 16
      end if
      !mark corners
      if (surface(is)%convexhull(i) > 0 .AND. &
          mod(surface(is)%convexhull(i), 6) == 0) then
         surface(is)%convexhull(i) = surface(is)%convexhull(i) + 1
      end if
   enddo

   !-------------------------------------------|
   !-----building icon array-------------------|
   !-------------------------------------------|
   ! we use the NN library to compute the triangulation
   ! of points for the surface
   ntmax=surface(is)%nsurface*3
   nhmax=surface(is)%nsurface
   npmax=surface(is)%nsurface
   nnpnmax=100
   nmax=3*ntmax+npmax
   nvmax=ntmax
   allocate (points(2,surface(is)%nsurface),stat=err)
   if (err.ne.0) call stop_run ('Error alloc points in create_surface')
   allocate (field(surface(is)%nsurface),stat=err)
   if (err.ne.0) call stop_run ('Error alloc field in create_surface')
   allocate (vertices(3,ntmax),stat=err)
   if (err.ne.0) call stop_run ('Error alloc vertices in create_surface')
   allocate (centres(3,ntmax),stat=err)
   if (err.ne.0) call stop_run ('Error alloc centress in create_surface')
   allocate (neighbour(3,ntmax),stat=err)
   if (err.ne.0) call stop_run ('Error alloc neighbour in create_surface')
   allocate (hulltriangles(nhmax),stat=err)
   if (err.ne.0) call stop_run ('Error alloc hulltriangles in create_surface')
   allocate (vis_tlist(nvmax),stat=err)
   if (err.ne.0) call stop_run ('Error alloc vis_tlist in create_surface')
   allocate (vis_elist(nvmax),stat=err)
   if (err.ne.0) call stop_run ('Error alloc vis_elist in create_surface')
   allocate (add_tlist(nvmax),stat=err)
   if (err.ne.0) call stop_run ('Error alloc add_tlist in icreate_surface')
   allocate (lt_work(ntmax),stat=err)
   if (err.ne.0) call stop_run ('Error alloc lt_work in create_surface')
   allocate (ln_work(surface(is)%nsurface),stat=err)
   if (err.ne.0) call stop_run ('Error alloc ln_work in create_surface')
   points(1,:)=surface(is)%x
   points(2,:)=surface(is)%y
   dmode=-2
   nmode=0
   loc=1
   nohalt_hull=0
   clockwise=.true.
   eps=1.d-8
   call nn2d_setup (surface(is)%nsurface,ntmax,nhmax,npmax,nnpnmax,nmax, &
                    points,dmode,nmode,clockwise,field,nt,vertices, &
                    centres,neighbour,nh,hulltriangles,nohalt_hull, &
                    loc,nnn,nnlist,ntrilist, &
                    eps,nvmax,vis_tlist,vis_elist,add_tlist, &
                    lt_work,ln_work)
   if (iproc.eq.0) write (8,*) 'surface ',is,' has ',nt,' triangles'

   surface(is)%icon(1:3,1:surface(is)%nt)=vertices(1:3,1:nt)

   deallocate (points)
   deallocate (field)
   deallocate (vertices)
   deallocate (centres)
   deallocate (neighbour)
   deallocate (hulltriangles)
   deallocate (vis_tlist)
   deallocate (vis_elist)
   deallocate (add_tlist)
   deallocate (lt_work)
   deallocate (ln_work)

   !----------------------------------------------|
   !----computing z coordinate--------------------|
   !----------------------------------------------|
   surface(is)%z = 0.0d0
   call zpoints (surface,is,params%ns, 0)

   !Scale with vertical exaggeration
   surface(is)%z = surface(is)%z/params%vex

   !----------------------------------------------|
   !----computing the normals---------------------|
   !----------------------------------------------|

   call compute_normals (surface(is)%nsurface,surface(is)%x,surface(is)%y,surface(is)%z, &
                         surface(is)%nt,surface(is)%icon, &
                         surface(is)%xn,surface(is)%yn,surface(is)%zn)

   if (params%debug>=2) call output_surf (surface(is),is,'init',-1,-1)

end if

end subroutine create_surf




!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine zpoints (surface,is,nsurf,nstack)

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! knowing the two-dimensional spatial distribution of points on the plane (x,y)
! this routine computes the z-coordinate accordingly to the surface type.
! surface are all surface sheets
! is is the surface for which z elevation will be calculate
! nsurf is the total number of surfaces
!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

use constants
use m_toolbox
use definitions

implicit none

type (sheet), dimension(nsurf) :: surface
integer is, nsurf, nstack

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|

integer :: i,iproc,k,kstart,kend
double precision :: dist,ran,m,x1a,x1b,x2a,x2b,y1a,y1b,y2a,y2b,zx,xsym
double precision :: psi,theta1,theta2,theta1s,theta2s,psis,y0s,y1s,y2s,x0,y0w
double precision :: y0a,y0b,y0,y1,y2,a1,b1,bb1,c1,m1,a2,b2,bb2,c2,m2,a3,b3,bb3
double precision :: c3,m3,a4,b4,bb4,c4,m4,eps,psi2,psi2s,psi3,psi3s,dz1,dz2,dz3
double precision :: dz4,psi1,psi1s,y0u,y0l,y1u,y1l,y2u,y2l,x0ut
double precision :: x0ub,x0lt,x0lb,x1ut,x1ub,x1lt,x1lb,x2ut,x2ub,x2lt,x2lb,y12mt
double precision :: y12mb,y2as,y2bs,y2au,y2al,y2bu,y2bl,x2aut,x2aub,x2alt,x2alb
double precision :: x2but,x2blt,x2bub,x2blb,y2rad

! used for case 15
double precision :: z0,xc,dh,phi,scalez
double precision :: yc1,h1,rx1,ry1,ryn1,rys1,r1,dist1
double precision :: yc2,rx2,ry2,ryn2,rys2,r2,dist2,z2


! used for case 16
double precision :: xAup, xAlo, xBup, xBlo, minxB, minxA

double precision, dimension (:), allocatable :: zprev

integer :: surface_type
double precision :: sp01,sp02,sp03,sp04,sp05,sp06,sp07,sp08,sp09,sp10,sp11,sp12
double precision :: sp13,sp14,sp15
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

if (surface(is)%stacking) then
    !allocate array for adding surfaces
    allocate (zprev(surface(is)%nsurface))
    if (nstack == 0) then
        zprev = 0.0d0
        !loop over all surfaces from current to last
        kstart = is 
        kend = nsurf
    else if (nstack > 0) then
        zprev = surface(is)%z
        !loop over all surfaces from one deeper to stacking depth
        kstart = is + 1
        kend = min(nsurf, is+nstack)
    end if
else
    kstart = is
    kend = is
end if


eps=1.d-10
iproc=0

!stacking loop: loop over all surfaces from current to highest number with the
!current surfaces's x and y coordinates (needed for random surfaces) and make
!sure the current surface is at least as high in z as all surfaces below.
do k = kstart,kend

!x, y, z and nsurface are always taken from current surface (surface(is)%x etc)
!surface_type and surface parameters sp* are iterated (surface(k)%sp01 etc)

surface_type = surface(k)%surface_type
sp01 = surface(k)%sp01
sp02 = surface(k)%sp02
sp03 = surface(k)%sp03
sp04 = surface(k)%sp04
sp05 = surface(k)%sp05
sp06 = surface(k)%sp06
sp07 = surface(k)%sp07
sp08 = surface(k)%sp08
sp09 = surface(k)%sp09
sp10 = surface(k)%sp10
sp11 = surface(k)%sp11
sp12 = surface(k)%sp12
sp13 = surface(k)%sp13
sp14 = surface(k)%sp14
sp15 = surface(k)%sp15

!save surface from previous iteration
!only occurs for multiple iterations if stacking = true
if (k>kstart) then
    zprev = surface(is)%z
end if

select case(surface_type)
   case (1)
        ! a flat surface,
        ! sp01 is the z level
        surface(is)%z=sp01
   case (2)
        ! rectangular emboss,
        ! sp01 is the z level
        ! sp02 and 03 are x1,x2
        ! sp04 and 05 are y1,y2
        ! sp06 is the thickness
        do i=1,surface(is)%nsurface
           if ((surface(is)%x(i)-sp02)*(surface(is)%x(i)-sp03).le.0.d0 .and. &
               (surface(is)%y(i)-sp04)*(surface(is)%y(i)-sp05).le.0.d0) then
              surface(is)%z(i)=sp01-sp06
           else
              surface(is)%z(i)=sp01
           end if
        end do
   case (3)
        ! convex spherical emboss,
        ! sp01 is the z level
        ! sp02 and 03 are x0,y0
        ! sp04 is the radius
        do i=1,surface(is)%nsurface
           dist=sqrt((surface(is)%x(i)-sp02)**2+(surface(is)%y(i)-sp03)**2)
           if (dist.le.sp04) then
              surface(is)%z(i)=sp01+sp04*sin(acos(dist/sp04))
           else
              surface(is)%z(i)=sp01
           endif
       end do
   case (4)
        ! convex spherical emboss,
        ! sp01 is the z level
        ! sp02 and 03 are x0,y0
        ! sp04 is the radius
        do i=1,surface(is)%nsurface
           dist=sqrt((surface(is)%x(i)-sp02)**2+(surface(is)%y(i)-sp03)**2)
           if (dist.le.sp04) then
              surface(is)%z(i)=sp01-sp04*sin(acos(dist/sp04))
           else
              surface(is)%z(i)=sp01
           endif
       end do
   case (5)
        ! double rectangular emboss,
        ! sp01 is the z level
        ! sp02 and 03 are x1,x2
        ! sp04 and 05 are x3,x4
        ! sp06 and 07 are y1,y2
        ! sp08 and 09 are y3,y4
        ! sp10 is the thickness
         surface(is)%z=sp01
         do i=1,surface(is)%nsurface
            if ((surface(is)%x(i)-sp02)*(surface(is)%x(i)-sp03).le.0.d0 .and.&
                (surface(is)%y(i)-sp06)*(surface(is)%y(i)-sp07).le.0.d0) then
               surface(is)%z(i)=surface(is)%z(i)-sp10
            endif
            if ((surface(is)%x(i)-sp04)*(surface(is)%x(i)-sp05).le.0.d0 .and.&
                (surface(is)%y(i)-sp08)*(surface(is)%y(i)-sp09).le.0.d0) then
              surface(is)%z(i)=surface(is)%z(i)-sp10
           endif
        end do
   case (6)
        ! a sinus,
        ! sp01 is the z level
        ! sp02 is the wavelength
        ! sp03 is the amplitude
        ! sp04 is the phase
        do i=1,surface(is)%nsurface
           surface(is)%z(i)=sp01+sp03*&
                sin(2.d0*pi*(surface(is)%x(i)/sp02+sp04))
        end do
   case (7)
        ! a noisy surface,
        ! sp01 is the z level
        ! sp02 is the noise amplitude
        do i=1,surface(is)%nsurface
           call random_number (ran)
           surface(is)%z(i)=sp01+ran*sp02
        end do
   case (8)
        ! a double sinus.
        ! sp01 is the z level
        ! sp02 is the x-wavelength
        ! sp03 is the x-amplitude
        ! sp04 is the y-wavelength
        ! sp05 is the y-amplitude
        do i=1,surface(is)%nsurface
           surface(is)%z(i)=sp01+ &
               sp03*sin(surface(is)%x(i)*2.*pi/sp02) + &
               sp05*sin(surface(is)%y(i)*2.*pi/sp04)
        end do
   case (9)
        ! a cosinus,
        ! sp01 is the z level
        ! sp02 is the wavelength
        ! sp03 is the amplitude
        ! sp04 is the phase
        do i=1,surface(is)%nsurface
           surface(is)%z(i)=sp01 + &
                sp03*cos(2.d0*pi*(surface(is)%x(i)/sp02+sp04))
        end do
   case (10)
        ! a 2D embankment
        ! sp01 is z0
        ! sp02 is y0
        ! sp03 is psi
        ! sp04 is the thickness
        do i=1,surface(is)%nsurface
           if (surface(is)%y(i)<sp02) then
              surface(is)%z(i)=sp01
           else
              surface(is)%z(i)=sp01 + &
                   min((surface(is)%y(i)-sp02)*tan(sp03*pi/180.d0),&
                   sp04)
           end if
        end do
   case (11)
       ! a 2D hill
       ! sp01 is z level
       ! sp02 height
       ! sp03 is y0
       ! sp04 is width
        do i=1,surface(is)%nsurface
           if (surface(is)%y(i)>sp03-sp04/2.d0 .and. &
              surface(is)%y(i)<sp03+sp04/2.d0) then
             surface(is)%z(i)=sp01 + &
                  sp02*(sin(pi*((surface(is)%y(i)-sp03)/&
                  sp04+0.5d0)))**2
           else
              surface(is)%z(i)=sp01
           end if
        end do
   case (12)
        ! rectangular emboss with specified slope
        ! sp01 is the z level
        ! sp02 and 03 are x1,x2
        ! sp04 and 05 are y1,y2
        ! sp06 is the thickness
        ! sp07 is the slope
        m=tan(sp07*pi/180.d0)
        m=sign(m,-sp06)
        x1a=sp02+(sp06/(2.d0*m))
        x1b=sp02-(sp06/(2.d0*m))
        x2a=sp03+(sp06/(2.d0*m))
        x2b=sp03-(sp06/(2.d0*m))
        y1a=sp04+(sp06/(2.d0*m))
        y1b=sp04-(sp06/(2.d0*m))
        y2a=sp05+(sp06/(2.d0*m))
        y2b=sp05-(sp06/(2.d0*m))
        do i=1,surface(is)%nsurface
           if (surface(is)%x(i).le.x1a) then
             zx=sp01
           elseif (surface(is)%x(i).le.x1b) then
             zx=sp01-(((surface(is)%x(i)-x1a)/(-sp06/m))*sp06)
           elseif (surface(is)%x(i).le.x2a) then
             zx=sp01-sp06
           elseif (surface(is)%x(i).le.x2b) then
             zx=sp01-((1.d0-((surface(is)%x(i)-x2a)/(-sp06/m)))*sp06)
             !zx=sp01-((1.d0-((surface(is)%x(i)-x2a)/(sp06/m)))*sp06)
           else
             zx=sp01
           endif
           ! There is a smarter way to do this, but...
           if (sp06.gt.eps) then
             if (surface(is)%y(i).le.y1a) then
               surface(is)%z(i)=sp01
             elseif (surface(is)%y(i).le.y1b) then
               surface(is)%z(i)=max(sp01-(((surface(is)%y(i)-y1a)/(-sp06/m))*sp06),zx)
             elseif (surface(is)%y(i).le.y2a) then
               surface(is)%z(i)=max(sp01-sp06,zx)
             elseif (surface(is)%y(i).le.y2b) then
               surface(is)%z(i)=max(sp01-((1.d0-((surface(is)%y(i)-y2a)/(-sp06/m)))*sp06),zx)
             else
               surface(is)%z(i)=sp01
             endif
           else
             if (surface(is)%y(i).le.y1a) then
               surface(is)%z(i)=sp01
             elseif (surface(is)%y(i).le.y1b) then
               surface(is)%z(i)=min(sp01-(((surface(is)%y(i)-y1a)/(-sp06/m))*sp06),zx)
             elseif (surface(is)%y(i).le.y2a) then
               surface(is)%z(i)=min(sp01-sp06,zx)
             elseif (surface(is)%y(i).le.y2b) then
               surface(is)%z(i)=min(sp01-((1.d0-((surface(is)%y(i)-y2a)/(-sp06/m)))*sp06),zx)
             else
               surface(is)%z(i)=sp01
             endif
           endif
        end do

   case (13)
        ! a 3D embankment of finite length and with 2 kinks
        ! sp01 is the z level
        ! sp02 is the z level 2
        ! sp03 is slope angle (psi)
        ! sp04 is x value for the center of the slope (x0)
        ! sp05 is the x width of the narrow end of the plateau (y0w)
        ! sp06 is the y position for the start of the slope (y0)
        ! sp07 is the y position of the first kink (y1)
        ! sp08 is the angle of the first kink (theta1)
        ! sp09 is the y position of the second kink (y2)
        ! sp10 is the angle of the second kink (theta2)
        ! sp11 is the reference elevation (elevation of the lower outer dip change)
        ! sp12 is the dip angle of the lower outer dip panel
        ! sp13 is the reference elevation (elevation of the upper outer flat)
        ! sp14 is the dip angle of the upper outer dip panel
        dz1=sp02-sp01
        !dz2=sp01-sp11
        dz2=sp01-sp11
        dz3=sp01-sp13
        dz4=sp11-sp13
        psi=sp03*pi/180.d0
        !psi2=sp12*pi/180.d0
        psi2=sp12*pi/180.d0
        psi3=sp14*pi/180.d0
        theta1=sp08*pi/180.d0
        theta2=sp10*pi/180.d0
        psis=sign(tan(psi),dz1)
        psi2s=sign(tan(psi2),dz2)
        psi3s=sign(tan(psi3),dz3)
        theta1s=tan(theta1)
        theta2s=tan(theta2)
        y0s=1.d0
        y1s=tan(theta1/2.d0)
        y2s=tan((theta1+theta2)/2.d0)
        x0=sp04

        y0w=sp05+dz1/(2.d0*psis)
        y0=sp06+dz1/(2.d0*psis)
        y0a=y0+dz1/(2.d0*psis)
        y0b=y0-dz1/(2.d0*psis)
        y1=sp07+(dz1/(2.d0*psis))*y1s
        y2=sp09+(dz1/(2.d0*psis))*y2s

        m1=((y2+(dz1/(2.d0*psis))*y1s)-(y1+(dz1/(2.d0*psis))*y1s))/&
           ((x0-y0w-(y1-y2)*theta1s-(dz1/(2.d0*psis)))-(x0-y0w-(dz1/(2.d0*psis))))
        bb1=(y1+(dz1/(2.d0*psis))*y1s)-m1*(x0-y0w-(dz1/(2.d0*psis)))
        a1=-m1
        b1=1
        c1=-bb1

        m2=((y2-(dz1/(2.d0*psis))*y1s)-(y1-(dz1/(2.d0*psis))*y1s))/&
           ((x0-y0w-(y1-y2)*theta1s+(dz1/(2.d0*psis)))-(x0-y0w+(dz1/(2.d0*psis))))
        bb2=(y1-(dz1/(2.d0*psis))*y1s)-m1*(x0-y0w+(dz1/(2.d0*psis)))
        a2=-m2
        b2=1
        c2=-bb2

        m3=((y2-(dz1/(2.d0*psis))*y1s)-(y1-(dz1/(2.d0*psis))*y1s))/&
           ((x0+y0w+(y1-y2)*theta1s-(dz1/(2.d0*psis)))-(x0+y0w-(dz1/(2.d0*psis))))
        bb3=(y1-(dz1/(2.d0*psis))*y1s)-m3*(x0+y0w-(dz1/(2.d0*psis)))
        a3=-m3
        b3=1
        c3=-bb3

        m4=((y2+(dz1/(2.d0*psis))*y1s)-(y1+(dz1/(2.d0*psis))*y1s))/&
           ((x0+y0w+(y1-y2)*theta1s+(dz1/(2.d0*psis)))-(x0+y0w+(dz1/(2.d0*psis))))
        bb4=(y1+(dz1/(2.d0*psis))*y1s)-m3*(x0+y0w+(dz1/(2.d0*psis)))
        a4=-m4
        b4=1
        c4=-bb4

        do i=1,surface(is)%nsurface
          surface(is)%z(i)=sp01
          if (surface(is)%y(i).ge.-y0s*surface(is)%x(i)+(y0+(x0-y0w)) .and. &
              surface(is)%y(i).ge.y0s*surface(is)%x(i)+(y0-(x0+y0w))) then
            if (surface(is)%y(i).ge.y0a) then
              if (surface(is)%y(i).ge.y0a+(dz2/psi2s)) then
                if (abs(dz4).gt.eps) then
                  if (dz4.gt.eps) then
                    surface(is)%z(i)=max(sp11-psi3s*(surface(is)%y(i)-(y0a+(dz2/psi2s))),sp13)
                  else
                    surface(is)%z(i)=min(sp11-psi3s*(surface(is)%y(i)-(y0a+(dz2/psi2s))),sp13)
                  endif
                else
                  surface(is)%z(i)=sp13
                endif
              else
                surface(is)%z(i)=sp01-psi2s*(surface(is)%y(i)-y0a)
              endif
             !if (dz2.gt.eps) then
               !surface(is)%z(i)=max(sp01-psi2s*(surface(is)%y(i)-y0a),sp11)
             !else
               !surface(is)%z(i)=min(sp01-psi2s*(surface(is)%y(i)-y0a),sp11)
             !endif
            elseif (surface(is)%y(i).ge.y0b) then
              surface(is)%z(i)=sp01+psis*(y0a-surface(is)%y(i))
            elseif (surface(is)%y(i).lt.y0b) then
              surface(is)%z(i)=sp02
            endif
          elseif (surface(is)%y(i).ge.-y1s*surface(is)%x(i)+(y1s*(x0-y0w)+y1) .and. &
                  surface(is)%y(i).ge.y1s*surface(is)%x(i)+(-y1s*(x0+y0w)+y1)) then
            if (surface(is)%x(i).lt.((x0-y0w)-(dz1/(2.d0*psis)))) then
              if (surface(is)%x(i).lt.((x0-y0w)-(dz1/(2.d0*psis)))-(dz2/psi2s)) then
                if (abs(dz4).gt.eps) then
                  if (dz4.gt.eps) then
                    surface(is)%z(i)=max(sp11-psi3s*(((x0-y0w)-(dz2/psi2s))-surface(is)%x(i)),sp13)
                  else
                    surface(is)%z(i)=min(sp11-psi3s*(((x0-y0w)-(dz2/psi2s))-surface(is)%x(i)),sp13)
                  endif
                else
                  surface(is)%z(i)=sp13
                endif
              else
                surface(is)%z(i)=sp01-psi2s*(((x0-y0w)-(dz1/(2.d0*psis)))-surface(is)%x(i))
              endif
             !if ((dz2).gt.eps) then
               !surface(is)%z(i)=max(sp01-psi2s*(((x0-y0w)-(dz1/(2.d0*psis)))-surface(is)%x(i)),sp11)
             !else
               !surface(is)%z(i)=min(sp01-psi2s*(((x0-y0w)-(dz1/(2.d0*psis)))-surface(is)%x(i)),sp11)
             !endif
            elseif (surface(is)%x(i).ge.((x0-y0w)-(dz1/(2.d0*psis))) .and. &
                    surface(is)%x(i).lt.((x0-y0w)+(dz1/(2.d0*psis)))) then
              surface(is)%z(i)=sp01+psis*(surface(is)%x(i)-((x0-y0w)-(dz1/(2.d0*psis))))
            elseif (surface(is)%x(i).ge.((x0-y0w)+(dz1/(2.d0*psis))) .and. &
                    surface(is)%x(i).lt.((x0+y0w)-(dz1/(2.d0*psis)))) then
              surface(is)%z(i)=sp02
            elseif (surface(is)%x(i).ge.((x0+y0w)-(dz1/(2.d0*psis))) .and. &
                    surface(is)%x(i).lt.((x0+y0w)+(dz1/(2.d0*psis)))) then
              surface(is)%z(i)=sp01-psis*(surface(is)%x(i)-((x0+y0w)+(dz1/(2.d0*psis))))
            else
              if (surface(is)%x(i).lt.((x0+y0w)+(dz1/(2.d0*psis)))+(dz2/psi2s)) then
                surface(is)%z(i)=sp01-psi2s*(surface(is)%x(i)-((x0+y0w)+(dz1/(2.d0*psis))))
              else
                if (abs(dz4).gt.eps) then
                  if (dz4.gt.eps) then
                    surface(is)%z(i)=max(sp11-psi3s*(surface(is)%x(i)-((x0+y0w)+(dz2/psi2s))),sp13)
                  else
                    surface(is)%z(i)=min(sp11-psi3s*(surface(is)%x(i)-((x0+y0w)+(dz2/psi2s))),sp13)
                  endif
                else
                  surface(is)%z(i)=sp13
                endif
              endif
             !if ((dz2).gt.eps) then
               !surface(is)%z(i)=max(sp01-psi2s*(surface(is)%x(i)-((x0+y0w)+(dz1/(2.d0*psis)))),sp11)
             !else
               !surface(is)%z(i)=min(sp01-psi2s*(surface(is)%x(i)-((x0+y0w)+(dz1/(2.d0*psis)))),sp11)
             !endif
            endif
          elseif (surface(is)%y(i).ge.-y2s*surface(is)%x(i)+( y2s*(x0-y0w-(y1-y2)*theta1s)+y2).and.&
                  surface(is)%y(i).ge. y2s*surface(is)%x(i)+(-y2s*(x0+y0w+(y1-y2)*theta1s)+y2)) then
            if (surface(is)%y(i).ge.m1*surface(is)%x(i)+bb1) then
              dist=abs(a1*surface(is)%x(i)+b1*surface(is)%y(i)+c1)/sqrt(a1**2.d0+b1**2.d0)
              if (dist.lt.(dz2/psi2s)) then
                surface(is)%z(i)=sp01-psi2s*dist
              else
                if (abs(dz4).gt.eps) then
                  if (dz4.gt.eps) then
                    surface(is)%z(i)=max(sp11-psi3s*(dist-(dz2/psi2s)),sp13)
                  else
                    surface(is)%z(i)=min(sp11-psi3s*(dist-(dz2/psi2s)),sp13)
                  endif
                else
                  surface(is)%z(i)=sp13
                endif
              endif
             !if ((dz2).gt.eps) then
               !surface(is)%z(i)=max(sp01-psi2s*dist,sp11)
             !else
               !surface(is)%z(i)=min(sp01-psi2s*dist,sp11)
             !endif
            elseif (surface(is)%y(i).ge.m2*surface(is)%x(i)+bb2 .and. &
                    surface(is)%y(i).lt.m1*surface(is)%x(i)+bb1) then
              dist=abs(a1*surface(is)%x(i)+b1*surface(is)%y(i)+c1)/sqrt(a1**2.d0+b1**2.d0)
              surface(is)%z(i)=sp01+dist*psis
            elseif (surface(is)%y(i).lt.m2*surface(is)%x(i)+bb2 .and. &
                    surface(is)%y(i).lt.m3*surface(is)%x(i)+bb3) then
              surface(is)%z(i)=sp02
            elseif (surface(is)%y(i).ge.m3*surface(is)%x(i)+bb3 .and.&
                    surface(is)%y(i).lt.m4*surface(is)%x(i)+bb4) then
              dist=abs(a4*surface(is)%x(i)+b4*surface(is)%y(i)+c4)/sqrt(a4**2.d0+b4**2.d0)
              surface(is)%z(i)=sp01+dist*psis
            else
              dist=abs(a4*surface(is)%x(i)+b4*surface(is)%y(i)+c4)/sqrt(a4**2.d0+b4**2.d0)
              if (dist.lt.(dz2/psi2s)) then
                surface(is)%z(i)=sp01-psi2s*dist
              else
                if (abs(dz4).gt.eps) then
                  if (dz4.gt.eps) then
                    surface(is)%z(i)=max(sp11-psi3s*(dist-(dz2/psi2s)),sp13)
                  else
                    surface(is)%z(i)=min(sp11-psi3s*(dist-(dz2/psi2s)),sp13)
                  endif
                else
                  surface(is)%z(i)=sp13
                endif
              endif
             !if ((dz2).gt.eps) then
               !surface(is)%z(i)=max(sp01-psi2s*dist,sp11)
             !else
               !surface(is)%z(i)=min(sp01-psi2s*dist,sp11)
             !endif
            endif
          elseif (surface(is)%x(i).lt.x0-y0w-(y1-y2)*theta1s-dz1/(2.d0*psis)) then
            if (surface(is)%x(i).lt.x0-y0w-(y1-y2)*theta1s-dz1/(2.d0*psis)-(dz2/psi2s)) then
              if (abs(dz4).gt.eps) then
                if (dz4.gt.eps) then
                  surface(is)%z(i)=max(sp11-psi3s*(((x0-y0w)-(y1-y2)*theta1s-(dz2/psi2s))-surface(is)%x(i)),sp13)
                  !surface(is)%z(i)=sp13
                else
                  surface(is)%z(i)=min(sp11-psi3s*(((x0-y0w)-(y1-y2)*theta1s-(dz2/psi2s))-surface(is)%x(i)),sp13)
                  !surface(is)%z(i)=sp13
                endif
              else
                surface(is)%z(i)=sp13
              endif
            else
              surface(is)%z(i)=sp01-psi2s*((x0-y0w-(y1-y2)*theta1s-dz1/(2.d0*psis))-&
                               surface(is)%x(i))
              !surface(is)%z(i)=sp13
            endif
           !if ((dz2).gt.eps) then
             !surface(is)%z(i)=max(sp01-psi2s*((x0-y0w-(y1-y2)*theta1s-dz1/(2.d0*psis))-&
                 !surface(is)%x(i)),sp11)
           !else
             !surface(is)%z(i)=min(sp01-psi2s*((x0-y0w-(y1-y2)*theta1s-dz1/(2.d0*psis))-&
                 !surface(is)%x(i)),sp11)
           !endif
          elseif (surface(is)%x(i).ge.((x0-y0w-(y1-y2)*theta1s)-(dz1/(2.d0*psis))) .and. &
                  surface(is)%x(i).lt.((x0-y0w-(y1-y2)*theta1s)+(dz1/(2.d0*psis)))) then
            surface(is)%z(i)=sp01+psis*(surface(is)%x(i)-((x0-y0w-(y1-y2)*theta1s)-&
                                 (dz1/(2.d0*psis))))
          elseif (surface(is)%x(i).ge.((x0-y0w-(y1-y2)*theta1s)+(abs(dz1)/(2.d0*psis))) .and. &
                  surface(is)%x(i).lt.((x0+y0w+(y1-y2)*theta1s)-(abs(dz1)/(2.d0*psis)))) then
            surface(is)%z(i)=sp02
          elseif (surface(is)%x(i).ge.((x0+y0w+(y1-y2)*theta1s)-(abs(dz1)/(2.d0*psis))) .and. &
                  surface(is)%x(i).lt.((x0+y0w+(y1-y2)*theta1s)+(abs(dz1)/(2.d0*psis)))) then
            surface(is)%z(i)=sp01-psis*(surface(is)%x(i)-((x0+y0w+(y1-y2)*theta1s)+&
                                 (dz1/(2.d0*psis))))
          else
            if (surface(is)%x(i).gt.x0+y0w+(y1-y2)*theta1s-dz1/(2.d0*psis)+(dz2/psi2s)) then
              if (abs(dz4).gt.eps) then
                if (dz4.gt.eps) then
                  surface(is)%z(i)=max(sp11-psi3s*(surface(is)%x(i)-(x0+y0w+(y1-y2)*theta1s+(dz2/psi2s))),sp13)
                else
                  surface(is)%z(i)=min(sp11-psi3s*(surface(is)%x(i)-(x0+y0w+(y1-y2)*theta1s+(dz2/psi2s))),sp13)
                endif
              else
                surface(is)%z(i)=sp13
              endif
            else
              !surface(is)%z(i)=sp01-psi2s*(surface(is)%x(i)-(x0-y0w-(y1-y2)*theta1s-dz1/(2.d0*psis)))
              surface(is)%z(i)=sp01-psi2s*(surface(is)%x(i)-&
                                   (x0+y0w+(y1-y2)*theta1s+dz1/(2.d0*psis)))
            endif
           !if ((dz2).gt.eps) then
             !surface(is)%z(i)=max(sp01-psi2s*(surface(is)%x(i)-(x0-y0w-(y1-y2)*theta1s-dz1/(2.d0*psis))),sp11)
           !else
             !surface(is)%z(i)=min(sp01-psi2s*(surface(is)%x(i)-(x0-y0w-(y1-y2)*theta1s-dz1/(2.d0*psis))),sp11)
           !endif
          endif
        enddo

   case (14)
        ! a 3D embankment of finite length and with rounded 2 kinks
        ! sp01 is the z level
        ! sp02 is the z level 2
        ! sp03 is slope angle (psi)
        ! sp04 is x value for the center of the slope (xsym)
        ! sp05 is the x width of the narrow end of the plateau (y0w)
        ! sp06 is the y position for the start of the slope (y0)
        ! sp07 is the y position of the first kink (y1)
        ! sp08 is the angle of the first kink (theta1)
        ! sp09 is the y position of the second kink (y2)
        ! sp10 is the angle of the second kink (theta2)
        ! sp11 is the reference elevation (elevation of the lower outer dip change)
        ! sp12 is the dip angle of the lower outer dip panel
        ! sp13 is the reference elevation (elevation of the upper outer flat)
        ! sp14 is the dip angle of the upper outer dip panel
        ! sp15 is the radius used for rounding the second kink

        ! Useful geometric values
        dz1=sp02-sp01
        dz2=sp01-sp11
        dz3=sp01-sp13
        dz4=sp11-sp13
        psi1=sp03*pi/180.d0
        psi2=sp12*pi/180.d0
        psi3=sp14*pi/180.d0
        theta1=sp08*pi/180.d0
        theta2=sp10*pi/180.d0
        y2rad=sp15
        psi1s=sign(tan(psi1),dz1)
        psi2s=sign(tan(psi2),dz2)
        psi3s=sign(tan(psi3),dz3)
        theta1s=tan(theta1)
        theta2s=tan(theta2)
        y0s=1.d0
        y1s=tan(theta1/2.d0)
        y2s=tan((theta1+theta2)/2.d0)

        xsym=sp04
        y0w=sp05
        y0=sp06
        y1=sp07
        y2=sp09

        y0u=y0
        y0l=y0+dz1/psi1s
        y1u=y1
        y1l=y1+(dz1/psi1s)*y1s
        y2u=y2
        y2l=y2+(dz1/psi1s)*y2s

        x0ut=xsym+y0w
        x0lt=xsym+y0w+dz1/psi1s
        x0ub=xsym-y0w
        x0lb=xsym-y0w-dz1/psi1s
        x1ut=x0ut
        x1lt=x0lt
        x1ub=x0ub
        x1lb=x0lb
        x2ut=x1ut+(y1-y2)*theta1s
        x2lt=x1lt+(y1-y2)*theta1s
        x2ub=x1ub-(y1-y2)*theta1s
        x2lb=x1lb-(y1-y2)*theta1s

        y12mt=(y2u-y1u)/(x2ut-x1ut)
        y12mb=(y2u-y1u)/(x2ub-x1ub)

        if (abs(theta1).gt.eps) then
            y2as=tan(pi/2.d0-theta1)
        else
            y2as=1.d0
        endif
        if (abs(theta2).gt.eps) then
            y2bs=tan(pi/2.d0-theta2)
        else
            y2bs=1.d0
        endif

        y2au=y2u+(y2rad*tan((theta1-theta2)/2.d0))*cos(theta1)
        if (abs(theta1).gt.eps) then
          y2al=y2u+(dz1/psi1s*y2as)
        else
          y2al=y2au
        endif
        y2bu=y2u-y2rad*tan((theta1-theta2)/2.d0)
        y2bl=y2bu

        x2aut=x2ut-(y2rad*tan((theta1-theta2)/2.d0))*y12mb
        if (abs(theta1).gt.eps) then
          x2alt=x2ut+(dz1/psi1s*(1.d0/y2as))-(y2rad*tan((theta1-theta2)/2.d0))*y12mb
        else
          x2alt=x2ut+dz1/psi1s-(y2rad*tan((theta1-theta2)/2.d0))*y12mb
        endif
        x2but=x2ut
        x2blt=x2lt
        x2aub=x2ub+(y2rad*tan((theta1-theta2)/2.d0))*y12mb
        if (abs(theta2).gt.eps) then
          x2alb=x2ub-(dz1/psi1s*(1.d0/y2as))+(y2rad*tan((theta1-theta2)/2.d0))*y12mb
        else
          x2alb=x2ub-dz1/psi1s+(y2rad*tan((theta1-theta2)/2.d0))*y12mb
        endif
        x2bub=x2ub
        x2blb=x2lb

        m1=(y2l-y1l)/(x2lb-x1lb)
        bb1=y1l-m1*x1lb
        a1=-m1
        b1=1
        c1=-bb1

        m2=(y2u-y1u)/(x2ub-x1ub)
        bb2=y1u-m2*x1ub
        a2=-m2
        b2=1
        c2=-bb2

        m3=(y2u-y1u)/(x2ut-x1ut)
        bb3=y1u-m3*x1ut
        a3=-m3
        b3=1
        c3=-bb3

        m4=(y2l-y1l)/(x2lt-x1lt)
        bb4=(y1l+(dz1/psi1s)*y1s)-m4*(x1lt+dz1/psi1s)
        a4=-m4
        b4=1
        c4=-bb4

        do i=1,surface(is)%nsurface
          surface(is)%z(i)=sp01
          ! Region "left" (y > plateau end) of lateral end of plateau
          if (surface(is)%y(i).ge.-y0s*surface(is)%x(i)+(y0u+x0ub) .and. surface(is)%y(i).ge.y0s*surface(is)%x(i)+(y0u-x0ut)) then
            if (surface(is)%y(i).ge.y0l) then
              if (surface(is)%y(i).ge.y0l+(dz2/psi2s)) then
                if (abs(dz4).gt.eps) then
                  if (dz4.gt.eps) then
                    surface(is)%z(i)=max(sp11-psi3s*(surface(is)%y(i)-(y0l+(dz2/psi2s))),sp13)
                  else
                    surface(is)%z(i)=min(sp11-psi3s*(surface(is)%y(i)-(y0l+(dz2/psi2s))),sp13)
                  endif
                else
                  surface(is)%z(i)=sp13
                endif
              else
                surface(is)%z(i)=sp01-psi2s*(surface(is)%y(i)-y0l)
              endif
            elseif (surface(is)%y(i).ge.y0u) then
              surface(is)%z(i)=sp01+psi1s*(y0l-surface(is)%y(i))
            elseif (surface(is)%y(i).lt.y0u) then
              surface(is)%z(i)=sp02
            endif
          ! Region "left" (y > plateau kink 1) oblique transition 1 of the plateau
          elseif (surface(is)%y(i).ge.-y1s*surface(is)%x(i)+(y1s*x1ub+y1u) .and. surface(is)%y(i).ge.y1s*surface(is)%x(i)+(-y1s*x1ut+y1u)) then
            if (surface(is)%x(i).lt.x1lb) then
              if (surface(is)%x(i).lt.x1lb-dz2/psi2s) then
                if (abs(dz4).gt.eps) then
                  if (dz4.gt.eps) then
                    surface(is)%z(i)=max(sp11-psi3s*((x1ub-dz2/psi2s)-surface(is)%x(i)),sp13)
                  else
                    surface(is)%z(i)=min(sp11-psi3s*((x1ub-dz2/psi2s)-surface(is)%x(i)),sp13)
                  endif
                else
                  surface(is)%z(i)=sp13
                endif
              else
                surface(is)%z(i)=sp01-psi2s*(x1lb-surface(is)%x(i))
              endif
            elseif (surface(is)%x(i).ge.x1lb .and. surface(is)%x(i).lt.x1ub) then
              surface(is)%z(i)=sp01+psi1s*(surface(is)%x(i)-x1lb)
            elseif (surface(is)%x(i).ge.x1ub .and. surface(is)%x(i).lt.x1ut) then
              surface(is)%z(i)=sp02
            elseif (surface(is)%x(i).ge.x1ut .and. surface(is)%x(i).lt.x1lt) then
              surface(is)%z(i)=sp01-psi1s*(surface(is)%x(i)-x1lt)
            else
              if (surface(is)%x(i).lt.x1lt+(dz2/psi2s)) then
                surface(is)%z(i)=sp01-psi2s*(surface(is)%x(i)-x1lt)
              else
                if (abs(dz4).gt.eps) then
                  if (dz4.gt.eps) then
                    surface(is)%z(i)=max(sp11-psi3s*(surface(is)%x(i)-x1lt),sp13)
                  else
                    surface(is)%z(i)=min(sp11-psi3s*(surface(is)%x(i)-x1lt),sp13)
                  endif
                else
                  surface(is)%z(i)=sp13
                endif
              endif
            endif
          ! Region "left" (y > arcuate region) of arcuate plateau margin
          elseif (surface(is)%y(i).ge.-y2as*surface(is)%x(i)+(y2as*x2aub+y2au) .and. surface(is)%y(i).ge.y2as*surface(is)%x(i)+(-y2as*x2aut+y2au)) then
            if (surface(is)%y(i).ge.m1*surface(is)%x(i)+bb1) then
              dist=abs(a1*surface(is)%x(i)+b1*surface(is)%y(i)+c1)/sqrt(a1**2.d0+b1**2.d0)
              if (dist.lt.(dz2/psi2s)) then
                surface(is)%z(i)=sp01-psi2s*dist
              else
                if (abs(dz4).gt.eps) then
                  if (dz4.gt.eps) then
                    surface(is)%z(i)=max(sp11-psi3s*(dist-(dz2/psi2s)),sp13)
                  else
                    surface(is)%z(i)=min(sp11-psi3s*(dist-(dz2/psi2s)),sp13)
                  endif
                else
                  surface(is)%z(i)=sp13
                endif
              endif
            elseif (surface(is)%y(i).ge.m2*surface(is)%x(i)+bb2 .and. &
                    surface(is)%y(i).lt.m1*surface(is)%x(i)+bb1) then
              dist=abs(a1*surface(is)%x(i)+b1*surface(is)%y(i)+c1)/sqrt(a1**2.d0+b1**2.d0)
              surface(is)%z(i)=sp01+dist*psi1s
            elseif (surface(is)%y(i).lt.m2*surface(is)%x(i)+bb2 .and. &
                    surface(is)%y(i).lt.m3*surface(is)%x(i)+bb3) then
              surface(is)%z(i)=sp02
            elseif (surface(is)%y(i).ge.m3*surface(is)%x(i)+bb3 .and. &
                    surface(is)%y(i).lt.m4*surface(is)%x(i)+bb4) then
              dist=abs(a4*surface(is)%x(i)+b4*surface(is)%y(i)+c4)/sqrt(a4**2.d0+b4**2.d0)
              surface(is)%z(i)=sp01+dist*psi1s
            else
              dist=abs(a4*surface(is)%x(i)+b4*surface(is)%y(i)+c4)/sqrt(a4**2.d0+b4**2.d0)
              if (dist.lt.(dz2/psi2s)) then
                surface(is)%z(i)=sp01-psi2s*dist
              else
                if (abs(dz4).gt.eps) then
                  if (dz4.gt.eps) then
                    surface(is)%z(i)=max(sp11-psi3s*(dist-(dz2/psi2s)),sp13)
                  else
                    surface(is)%z(i)=min(sp11-psi3s*(dist-(dz2/psi2s)),sp13)
                  endif
                else
                  surface(is)%z(i)=sp13
                endif
              endif
            endif
          ! Arcuate region of plateau
          elseif (surface(is)%y(i).ge.y2bu) then
            dist=sqrt((surface(is)%x(i)-(x2bub+y2rad))**2.d0+(surface(is)%y(i)-y2bu)**2.d0)
            if (dist <= y2rad) then
              surface(is)%z(i)=sp02
            elseif (dist <= y2rad+dz1/psi1s) then
              surface(is)%z(i)=sp01+((y2rad+dz1/psi1s)-dist)*psi1s
            elseif (dist <= y2rad+dz1/psi1s+dz2/psi2s) then
              surface(is)%z(i)=sp01-psi2s*(dist-(y2rad+dz1/psi1s))
            else
              if (abs(dz4) > eps) then
                if (dz4 > eps) then
                  surface(is)%z(i)=max(sp11-psi3s*(dist-(y2rad+dz1/psi1s+dz2/psi2s)),sp13)
                else
                  surface(is)%z(i)=min(sp11-psi3s*(dist-(y2rad+dz1/psi1s+dz2/psi2s)),sp13)
                endif
              else
                surface(is)%z(i)=sp13
              endif
            endif
          ! Initial linear plateau margin along symmetry plane
          elseif (surface(is)%x(i) < x2lb) then
            if (surface(is)%x(i) < x2lb-(dz2/psi2s)) then
              if (abs(dz4).gt.eps) then
                if (dz4.gt.eps) then
                  surface(is)%z(i)=max(sp11-psi3s*(x2lb-(dz2/psi2s)-surface(is)%x(i)),sp13)
                else
                  surface(is)%z(i)=min(sp11-psi3s*(x2lb-(dz2/psi2s)-surface(is)%x(i)),sp13)
                endif
              else
                surface(is)%z(i)=sp13
              endif
            else
              surface(is)%z(i)=sp01-psi2s*(x2lb-surface(is)%x(i))
            endif
          elseif (surface(is)%x(i).ge.x2lb .and. surface(is)%x(i).lt.x2ub) then
            surface(is)%z(i)=sp01+psi1s*(surface(is)%x(i)-x2lb)
          elseif (surface(is)%x(i).ge.x2ub .and. surface(is)%x(i).lt.x2ut) then
            surface(is)%z(i)=sp02
          elseif (surface(is)%x(i).ge.x2ut .and. surface(is)%x(i).lt.x2lt) then
            surface(is)%z(i)=sp01+psi1s*(x2lb-surface(is)%x(i))
          else
            if (surface(is)%x(i) > x2ut+(dz2/psi2s)) then
              if (abs(dz4).gt.eps) then
                if (dz4.gt.eps) then
                  surface(is)%z(i)=max(sp11-psi3s*(surface(is)%x(i)-(x2ut+(dz2/psi2s))),sp13)
                else
                  surface(is)%z(i)=min(sp11-psi3s*(surface(is)%x(i)-(x2ut+(dz2/psi2s))),sp13)
                endif
              else
                surface(is)%z(i)=sp13
              endif
            else
              surface(is)%z(i)=sp01-psi2s*(surface(is)%x(i)-x2lt)
            endif
          endif
        enddo

   case (15)
        ! sp01 is the z base level (z0)
        ! sp02 is the x position of the peak (xc)
        ! sp03 is the y position of the peak (yc1)
        !     For y<y0, the indenter will be a tube of shape as for y=y0
        ! sp04 is the elevation of the first indenter peak above base level (h1)
        ! sp05 is the first indenter x radius (distance from peak to base) (rx1)
        ! sp06 is the y radius (distance from peak to base) for the northern
        !     half of the first indenter (ryn1). Negative values create a
        !     cylinder extending to the y=1 edge.
        ! sp07 is the y width (distance from peak to base) for the southern
        !     half of the first indenter (ryn2). Negative values create a
        !     cylinder extending to the y=0 edge.
        ! sp08-sp11 describe y peak position (sp08=yc2), x radius (sp09=rx2),
        !     northern y radius (sp10=ryn2) and southern y radius (sp11=rys2)
        !     for a second indenter with identical axis of rotation. h2 will
        !     be calculated to ensure this. Set sp09<0 to disable second indenter.
        ! sp12 is a radial offset from the specified surface (for spherical shells)
        ! sp13 is the angle of slope in x-direction at the base in degree (phi)
        !     note that for ry<rx the slope at the base will be steeper in y direction
        !     also ensure that rx > h/tan(phi)
        !     phi<0 creates a dome unscaled in z-direction (doesn't force angle)

        z0 = sp01
        xc = sp02
        yc1 = sp03
        h1 = sp04
        rx1 = sp05
        ryn1 = sp06
        rys1 = sp07
        yc2 = sp08
        rx2 = sp09
        ryn2 = sp10
        rys2 = sp11
        dh = sp12
        phi = sp13


        a1=h1/tan(phi*pi/180.d0)
        if (rx1<=2*a1) then
            phi=-1
        end if

        if (phi>0) then
            !Radius of spere unscaled in z direction
            r1=rx1*(rx1-a1)/sqrt(rx1*(rx1-2*a1))
            !scaling factor for z-direction
            scalez = h1/(r1-sqrt(r1**2-rx1**2))
        else
            !Radius of unscaled sphere cutting through (xc,z0+h) and (xc+xr,z0)
            r1=(h1**2+rx1**2)/(2*h1)
            scalez=1
        end if
        !z-value of sphere center
        zx=z0+h1-r1*scalez

        !adjust radius for shell offset
        r1 = r1 + dh/scalez

        !Set second indenter radius if defined then
        if (rx2 > eps) then
            r2 = sqrt( rx2**2 + (scalez*(z0-zx))**2) + dh/scalez
        end if

        do i=1,surface(is)%nsurface
            !cylindrical tube behind xc
            if (surface(is)%x(i) < xc) then
                x0 = xc
            else
                x0 = surface(is)%x(i)
            end if

            !Set y radius for northern and southern part
            if (surface(is)%y(i) > yc1) then
                ry1 = ryn1
            else
                ry1 = rys1
            end if

            !if width is negative, set as cylinder then
            if (ry1 <= 0.d0) then
                y1 = 0.0
            else
                y1 = max(0.0,abs(surface(is)%y(i)-yc1) * rx1/ry1)
            end if

            dist1=sqrt((x0-xc)**2+y1**2)
            if (r1 < dist1) then
                surface(is)%z(i) = z0
            else
                surface(is)%z(i)=max(z0,zx+scalez*sqrt(r1**2-dist1**2))
            end if

            if (rx2 > eps) then
                !Set y radius for northern and southern part
                if (surface(is)%y(i) > yc2) then
                    ry2 = ryn2
                else
                    ry2 = rys2
                end if

                !if width is negative, set as cylinder then
                if (ry2 < eps) then
                    y2 = 0.0
                else
                    y2 = max(0.0,abs(surface(is)%y(i)-yc2) * rx2/ry2)
                end if

                dist2=sqrt((x0-xc)**2+y2**2)
                if (r2 < dist2) then
                    z2 = z0
                else
                    z2=max(z0,zx+scalez*sqrt(r2**2-dist2**2))
                end if
                surface(is)%z(i) = max(surface(is)%z(i), z2)
            end if
        end do

   case (16)
        ! sp01 = z1, left hand side z level
        ! sp02 = z2, right hand side z level
        ! sp03 = x0, x coord of corner point
        ! sp04 = y0, y coord of corner point
        ! sp05 = alpha1, 0 < alpha1 < 90deg
        ! sp06 = alpha2
        ! sp07 = theta, 0 <= theta < 90deg
        ! sp08 = theta_switch, <0 = theta is perpendicular to strike
        !                      >0 = theta is perpendicular to y axis

        do i=1,surface(is)%nsurface
          xAup = sp03 - (surface(is)%y(i) - sp04)*tan(pi*sp05/180)
          xAlo = sp03 - (sp04 - surface(is)%y(i))*tan(pi*sp06/180)
          xBup = xAup + (sp01-sp02)/(cos(pi*sp05/180)*tan(pi*sp07/180))
          xBlo = xAlo + (sp01-sp02)/(cos(pi*sp06/180)*tan(pi*sp07/180))
          minxB = min(xBup, xBlo)
          minxA = min(xAup, xAlo)

          if (surface(is)%x(i) <= minxA) then
             ! left plateau
             surface(is)%z(i) = sp01
          elseif (surface(is)%x(i) <= minxB) then
             ! ramp
             surface(is)%z(i) = sp01 + (surface(is)%x(i) - minxA) * (sp02 - sp01) / (minxB - minxA)
          else
             ! right hand side lower level
             surface(is)%z(i) = sp02
          endif
        enddo


   case (17)
        ! basin with differing height basement steps, geometry constant in y
        ! sp01 is the z level at the bottom of the basin
        ! sp02 and 03 are x1,x2
        ! sp04 is not used (this could specify a 2nd slope later)
        ! sp05 is the distal basement step thickness
        ! sp06 is the proximal basement step thickness
        ! sp07 is the slope
        m=tan(sp07*pi/180.d0)
        m=sign(m,-sp06)
        x1a=sp02+(sp06/(2.d0*m))
        x1b=sp02-(sp06/(2.d0*m))
        x2a=sp03+(sp05/(2.d0*m))
        x2b=sp03-(sp05/(2.d0*m))
        !y1a=sp04+(sp06/(2.d0*m))
        !y1b=sp04-(sp06/(2.d0*m))
        !y2a=sp05+(sp06/(2.d0*m))
        !y2b=sp05-(sp06/(2.d0*m))
        do i=1,surface(is)%nsurface
          if (surface(is)%x(i).le.x1a) then
            surface(is)%z(i)=sp01+sp06
          elseif (surface(is)%x(i).le.x1b) then
            surface(is)%z(i)=(sp01+sp06)-&
                 (((surface(is)%x(i)-x1a)/(-sp06/m))*sp06)
          elseif (surface(is)%x(i).le.x2a) then
            surface(is)%z(i)=sp01
          elseif (surface(is)%x(i).le.x2b) then
            surface(is)%z(i)=(sp01+sp05)-&
                 ((1.d0-((surface(is)%x(i)-x2a)/(-sp05/m)))*sp05)
            !zx=sp01-((1.d0-((surface(is)%x(i)-x2a)/(sp06/m)))*sp06)
          else
            surface(is)%z(i)=sp01+sp05
          endif
          !! There is a smarter way to do this, but...
          !if (sp06.gt.eps) then
          !  if (surface(is)%y(i).le.y1a) then
          !    surface(is)%z(i)=sp01
          !  elseif (surface(is)%y(i).le.y1b) then
          !    surface(is)%z(i)=max(sp01-(((surface(is)%y(i)-y1a)/(-sp06/m))*sp06),zx)
          !  elseif (surface(is)%y(i).le.y2a) then
          !    surface(is)%z(i)=max(sp01-sp06,zx)
          !  elseif (surface(is)%y(i).le.y2b) then
          !    surface(is)%z(i)=max(sp01-((1.d0-((surface(is)%y(i)-y2a)/(-sp06/m)))*sp06),zx)
          !  else
          !    surface(is)%z(i)=sp01
          !  endif
          !else
          !  if (surface(is)%y(i).le.y1a) then
          !    surface(is)%z(i)=sp01
          !  elseif (surface(is)%y(i).le.y1b) then
          !    surface(is)%z(i)=min(sp01-(((surface(is)%y(i)-y1a)/(-sp06/m))*sp06),zx)
          !  elseif (surface(is)%y(i).le.y2a) then
          !    surface(is)%z(i)=min(sp01-sp06,zx)
          !  elseif (surface(is)%y(i).le.y2b) then
          !    surface(is)%z(i)=min(sp01-((1.d0-((surface(is)%y(i)-y2a)/(-sp06/m)))*sp06),zx)
          !  else
          !    surface(is)%z(i)=sp01
          !  endif
          !endif
        end do

   case (18)
        ! JA 050415. Sinusoidally-perturbed surface, to match aggradation options
        ! sp01 is the z level at the base of the sinusoid
        ! sp02 and 03 are x1,x2
        ! sp04 05 are y1, y2
        ! sp06 is the wavelength in x
        ! sp07 is the amplitude x
        ! sp08 is the wavelength in y
        ! sp09 is the amplitude in y
        do i=1,surface(is)%nsurface

          if (surface(is)%x(i).ge.sp02.and.surface(is)%x(i).le.sp03.and.&
              surface(is)%y(i).ge.sp04.and.surface(is)%y(i).le.sp05) then
             if (sp09.lt.eps) then
                surface(is)%z(i)=sp01+sp07*&
                     (1+sin((surface(is)%x(i)-sp02)*pi/(sp06/2)-pi/2.d0))
             else
                surface(is)%z(i)=sp01+sp07*&
                     (1+sin((surface(is)%x(i)-sp02)*pi/(sp06/2)-pi/2.d0))*&
                     sp09*(1+sin((surface(is)%y(i)-sp04)*pi/(sp08/2.d0)-pi/2.d0))
             endif
          endif
        end do

   case (19)
        ! JA 070415. half-Gaussian surface, with the option of a Gaussian deformation
        ! in y. Matches progradation options
        ! sp01 and 02 are the z levels at the landward and seaward edges of the progradation profile
        ! sp03 and 04 are x1,x2
        ! sp05 is the half width of the Gaussian in x
        ! sp06 sets whether there will be a Gaussian in y (if 1)
        ! sp07 is the centre position of the Gaussian in y
        ! sp08 is the half width of the Gaussian in y

        do i=1,surface(is)%nsurface

          if (surface(is)%x(i).le.sp03) then
            if(sp06.gt.0) then
                surface(is)%z(i)=sp02+(sp01-sp02)*&
                     exp(-((surface(is)%y(i)-sp07)**2)/(2*sp08**2))
            else
              surface(is)%z(i)=sp01
            endif
          elseif(surface(is)%x(i).ge.sp03.and.surface(is)%x(i).le.sp04) then
            if(sp06.gt.0) then
              surface(is)%z(i)=sp02+(sp01-sp02)*&
                   exp(-((surface(is)%x(i)-sp03)**2)/(2*sp05**2))*&
                   exp(-((surface(is)%y(i)-sp07)**2)/(2*sp08**2))
            else
             surface(is)%z(i)=sp02+(sp01-sp02)*&
                  exp(-((surface(is)%x(i)-sp03)**2)/(2*sp05**2))
            endif
          elseif (surface(is)%x(i).ge.sp04) then
            surface(is)%z(i)=sp02
          endif

        end do

   case default
         call stop_run ('surface type not defined')
end select

!if second or higher iteration of this loop, shift surfaces up
!done by elementwise max
!nstack == 0is true for call at initalisation
!nstack > 0 is ture for call from move_surface during run
if (surface(is)%stacking .AND. &
    (((nstack == 0) .AND. (k > kstart)) .OR. (nstack > 0))) then
    do i = 1,surface(is)%nsurface
        surface(is)%z(i) = max(surface(is)%z(i),zprev(i))
    end do
end if

end do

if (surface(is)%stacking) then
    deallocate (zprev)
end if

return
end subroutine zpoints
end module m_create_surfaces
