module m_define_bc_rot_subduction
contains
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              DEFINE_BC_ROT_SUBDUCTION     AUG. 2014                          |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine define_bc_rot_subduction(params,osolve,vo,bcdef)

!------------------------------------------------------------------------------|
!((((((((((((((((  Purpose of the routine  )))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! This routine assigns the velocity boundary conditions for the rotational     |
! subduction geometry                                                          |
!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

use definitions
!use mpi

implicit none

include 'mpif.h' 

type (parameters) params
type (octreesolve) osolve
type (void) vo
type (bc_definition) bcdef

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|
logical :: horzflow, fixindrot, fixindtemp, freeindbot, horzsides, botoutflux
integer :: tempcase,restrict
double precision :: xc,yc1,h1,rx1,ryn1,rys1,yc2,rx2,ryn2,rys2,vin,vtop,vback,nelem,temp2

logical :: cuty0face,cuty1face,cutx1face,singletrans
integer :: i,iproc,nproc,ierr
double precision :: eps,pi,dxy,dz
double precision :: x0,y1,y2,z0,zc,rmain1,rmain2,h2,omega,theta
double precision :: ymaxn1,ymaxs1,scalemax1
double precision :: distmain1,distmainy1,disttrans1,distzoney1
double precision :: ymaxn2,ymaxs2,scalemax2
double precision :: distmain2,distmainy2,disttrans2,distzoney2
double precision :: distbot1, distbot2, disttransbot1, disttransbot2
double precision :: distrot,disttrans,distzoney,disttransbot
double precision :: xbotedgey0, xbotedgey1, xbotedgey02, xbotedgey12
double precision :: vx,vz,vx_above,tstart
double precision :: sublength, distsubzone, wsub, wsubend

!!For debugging
!double precision,dimension(:),allocatable :: debug1, debug2, debug3, debug4
!integer :: k, nnn, nne, nnne, ie, iconmin, iconmax
!integer :: myicon(100)
!logical debugging

!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

!Name conventions:
! xc, zc apply to coordinates of center of spheres - identical for both identers
! *0 are variables that apply to both indenters
! *1 and *2 only apply to the respective indenters

call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)

!debugging=.true.
!debugging=(iproc == 0 .and. debugging .and. params%first_step .and. params%first_iter)
!if (debugging) then
    !allocate(debug1(osolve%nnode))
    !allocate(debug2(osolve%nnode))
    !allocate(debug3(osolve%nnode))
    !allocate(debug4(osolve%nnode))
    !
    !debug1=0.d0
    !debug2=0.d0
    !debug3=0.d0
    !debug4=0.d0
!end if

!basic constants
eps=1.d-10
pi=atan(1.d0)*4.d0

!set fixed to 0 everywhere
osolve%kfix=0
osolve%kfixt=0


!parameters from input file 
xc=bcdef%bc_parameters(1)
yc1=bcdef%bc_parameters(2)
h1=bcdef%bc_parameters(3)
rx1=bcdef%bc_parameters(4)
ryn1=bcdef%bc_parameters(5)
rys1=bcdef%bc_parameters(6)
yc2=bcdef%bc_parameters(7)
rx2=bcdef%bc_parameters(8)
ryn2=bcdef%bc_parameters(9)
rys2=bcdef%bc_parameters(10)
vin=bcdef%bc_parameters(11)
vtop=bcdef%bc_parameters(12)
vback=-bcdef%bc_parameters(13) !change in orientation
tempcase=idint(bcdef%bc_parameters(14))
temp2=bcdef%bc_parameters(15)
nelem=bcdef%bc_parameters(16)
restrict=idint(bcdef%bc_parameters(17))

!decompose restrictions
horzflow = btest(restrict,0) 
fixindrot = btest(restrict,1)
fixindtemp = btest(restrict,2)
freeindbot = btest(restrict,3)
horzsides = btest(restrict,4)
!botoutflux only enbaled if 2nd indenter is defined
botoutflux = (btest(restrict,5) .AND. rx2 > eps)

!check and adjust restrictions for consistency
!unconstrained indenter bottom velocity unsets velocity in domain
if(freeindbot .AND. fixindrot) then
    fixindrot = .false.
    fixindtemp = .false.
end if
!adjust bottom temperature case
if(fixindtemp) then
    tempcase = 1
endif

dxy=nelem/2.d0**(params%levelmax_oct) !transition width in xy
dz=nelem*params%vex/2.d0**(params%levelmax_oct) !transition width in z

!Derived parameters from input values
rmain1=(h1**2+rx1**2)/(2.d0*h1)
zc=h1-rmain1
omega=vin/rmain1

!Transition zone ensures gradual change in velocity and temperature boundary
!conditions. Transition will be calculated as a liner gradient along the direction
!of the sphere center.
if (ryn1 > 0.d0) then
    ymaxn1 = sqrt( (min(0.d0,xc))**2 + ((ryn1+dxy)/ryn1*rx1)**2 + zc**2) / rmain1
else
    ymaxn1 = 1.d0
end if

if (rys1 > 0.d0) then
    ymaxs1 = sqrt( (min(0.d0,xc))**2 + ((rys1+dxy)/rys1*rx1)**2 + zc**2) / rmain1
else
    ymaxs1 = 1.d0
end if

!Disable side scales when second indenter exists
if (rx2 > eps) then
    if (ryn2 > ryn1 .OR. ryn2 < 0.d0) ymaxn1 = 1.d0
    if (rys2 > rys1 .OR. rys2 < 0.d0) ymaxs1 = 1.d0
end if

scalemax1 = max( (h1+dz-zc)/rmain1, ymaxn1, ymaxs1, & 
                sqrt((rx1+dxy)**2+zc**2)/rmain1) - 1.d0

!horzflow will only be used at those faces the indenter does not cut.
cuty0face = (rys1 < 0.d0 .or. yc1-rys1 < 0.d0)
cuty1face = (ryn1 < 0.d0 .or. yc1+ryn1 > 1.d0)
cutx1face = (xc+rx1 > 1.0d0) 

!x value of the indenter intersecting z=0 and y=0/y=1 edge
if (rys1 < 0.d0) then
    xbotedgey0 = xc + rx1
else
    xbotedgey0 =  max(0.d0, sqrt( rmain1**2 - (yc1 * rx1 / rys1)**2 - zc**2) + xc)
end if
if (ryn1 < 0.d0) then
    xbotedgey0 = xc + rx1
else
    xbotedgey1 =  max(0.d0, sqrt( rmain1**2 - ((1.d0 - yc1) * rx1 / ryn1)**2 - zc**2) + xc)
end if

!Calculate Values for second indenter, if defined
if (rx2 > eps) then
    rmain2 = sqrt(rx2**2 + zc**2)  
    h2 = zc + rmain2

    if (ryn2 > 0.d0) then
        ymaxn2 = sqrt( (min(0.d0,xc))**2 + ((ryn2+dxy)/ryn2*rx2)**2 + zc**2) / rmain2
    else
        ymaxn2 = 1.d0
    end if
    if (rys2 > 0.d0) then
        ymaxs2 = sqrt( (min(0.d0,xc))**2 + ((rys2+dxy)/rys2*rx2)**2 + zc**2) / rmain2
    else
        ymaxs2 = 1.d0
    end if

    !Disable side scales when first indenter is larger
    if (ryn2 < ryn1 .OR. ryn1 < 0.d0) ymaxn2 = 1.d0
    if (rys2 < rys1 .OR. rys1 < 0.d0) ymaxs2 = 1.d0

    scalemax2 = max( (h2+dz-zc)/rmain2, ymaxn2, ymaxs2, & 
                    sqrt((rx2+dxy)**2+zc**2)/rmain2) - 1.d0

    !Adjust scalemax1 and 2 so that both are the same (larger value)
    if (scalemax2*rmain2 > scalemax1*rmain1) then
        scalemax1 = scalemax2*rmain2 / rmain1
    else
        scalemax2 = scalemax1*rmain1 / rmain2
    end if

    !horzflow will only be used at those faces the indenter does not cut.
    cuty0face = cuty0face .or. (rys2 < 0.d0 .or. yc2-rys2 < 0.d0)
    cuty1face = cuty1face .or. (ryn2 < 0.d0 .or. yc2+ryn2 > 1.d0)
    cutx1face = cutx1face .or. (xc+rx2 > 1.0d0) 

    !x value of the indenter intersecting z=0 and y=0/y=1 edge
    if (rys2 < 0.d0) then
        xbotedgey02 = xc + rx2
    else
        xbotedgey02 =  max(0.d0, sqrt( rmain2**2 - (yc2 * rx2 / rys2)**2 - zc**2) + xc)
    end if
    if (ryn2 < 0.d0) then
        xbotedgey12 = xc + rx2
    else
        xbotedgey12 =  max(0.d0, sqrt( rmain2**2 - ((1.d0 - yc2) * rx2 / ryn2)**2 - zc**2) + xc)
    end if
    xbotedgey0 = max(xbotedgey0,  xbotedgey02)
    xbotedgey1 = max(xbotedgey1,  xbotedgey12)
end if

if (botoutflux) then
    !sublength is distance from end of transition zone of indenter2
    sublength = 1.d0 - sqrt( ((1.d0 + scalemax2) * rmain2)**2 - zc**2) - xc
    wsub = (2.d0 * vback * h2) / sublength  !vback is negative, so is wsub
end if

!Adjusting coordinates: The region behind the indenter (x<xc) shall be an 
!extension of the geometry at x=xc.

do i=1,osolve%nnode
    !Transforming coordinates to unit sphere:
    x0 = max(0.d0,(osolve%x(i) - xc))
    z0 = osolve%z(i)*params%vex-zc

    if (osolve%y(i) >= yc1) then
        y1 = max(0.d0,(osolve%y(i) - yc1)/ryn1*rx1)
    else
        y1 = max(0.d0,(yc1 - osolve%y(i))/rys1*rx1)
    end if

    !distance from center of sphere
    distmain1=sqrt(x0**2 + y1**2 + z0**2)
    !distance from axis of rotation
    distrot=sqrt(x0**2 + z0**2)
    !distance along transition zone
    disttrans1 = max(0.d0, min(1.d0, (distmain1/rmain1-1.d0)/scalemax1))

    !distzoney will be calculated for x=0 and z=0
    distmainy1=sqrt((max(0.d0,-xc))**2 + y1**2 + zc**2)
    distzoney1 = max(0.d0, min(1.d0, (distmainy1/rmain1-1.d0)/scalemax1))

    !distbot is distance from center for z=0
    distbot1 = sqrt(x0**2 + y1**2 + zc**2) 
    disttransbot1 = max(0.d0, (distbot1/rmain1-1.d0)/scalemax1)

    doubleindenter: if (rx2 > eps) then
    !both indenters defined
        if (osolve%y(i) >= yc2) then
            y2 = max(0.d0,(osolve%y(i) - yc2)/ryn2*rx2)
        else
            y2 = max(0.d0,(yc2 - osolve%y(i))/rys2*rx2)
        end if

        !distance from center of sphere
        distmain2=sqrt(x0**2 + y2**2 + z0**2)
        !distance along transition zone
        disttrans2 = max(0.d0, min(1.d0, (distmain2/rmain2-1.d0)/scalemax2))

        !distzoney will be calculated for x=0 and z=0
        distmainy2=sqrt((max(0.d0,-xc))**2 + y2**2 + zc**2)
        distzoney2 = max(0.d0, min(1.d0, (distmainy2/rmain2-1.d0)/scalemax2))

        !distbot is distance from center for z=0
        distbot2 = sqrt(x0**2 + y2**2 + zc**2) 
        disttransbot2 = max(0.d0, (distbot2/rmain2-2.d0)/scalemax2)
        
        disttrans = disttrans1*disttrans2
        distzoney = distzoney1*distzoney2
        disttransbot = disttransbot1 * disttransbot2

        !calculate velocity for indenter
        if (disttrans < 1.d0-eps) then
            !set rotation distance to minimum sphere if inside transition zone
            if (disttrans > eps) then
                singletrans = .false.
                !in indenter 1 transition zone
                if (disttrans2 > 1.d0-eps) then
                    distrot = sqrt(rmain1**2-y1**2)
                    singletrans = .true.
                end if
                !in indenter 2 transition zone
                if (disttrans1 > 1.d0-eps) then
                    distrot = sqrt(rmain2**2-y2**2)
                    singletrans = .true.
                end if
                !if in both transition zone
                if (.not. singletrans) then
                    !not exact, but good enough
                    distrot = (sqrt(rmain1**2-y1**2) + sqrt(rmain2**2-y2**2)) * 0.5d0
                end if
            end if
            
            theta=asin(x0/distrot)
            vx=distrot*omega*cos(theta)
            vz=-x0*omega
        end if

    else doubleindenter
    !only first indenter defined
        disttrans = disttrans1
        distzoney = distzoney1
        !if (debugging) disttrans2 = 1.5d0

        !calculate velocity for indenter
        if (disttrans < 1.d0-eps) then
            !set rotation distance to minimum sphere if inside transition zone
            if (disttrans > eps) then
                distrot = sqrt(rmain1**2-y1**2)
            end if
            
            theta=asin(x0/distrot)
            vx=distrot*omega*cos(theta)
            vz=-x0*omega
        end if
    end if doubleindenter
    
    !setting T and velocity in indenter, only below transition zone
    if (disttrans < eps) then
        !set velocity
        if (fixindrot) then
            !set fix flag
            osolve%kfix((i-1)*3+1)=1
            osolve%kfix((i-1)*3+2)=1
            osolve%kfix((i-1)*3+3)=1

            !assign velocity
            osolve%u(i) = vx
            osolve%v(i) = 0.d0
            osolve%w(i) = vz
        end if

        !set temperature
        if (fixindtemp) then
            osolve%kfixt(i)=1 
            osolve%temp(i)=temp2
        end if
    end if


    !y=0 face free slip 
    if (osolve%y(i)<eps) then
        !no flux through face
        osolve%kfix((i-1)*3+2)=1 ; osolve%v(i)=0.d0
        
        !rotating sphere cuts y=0 face
        if (cuty0face) then
            if (disttrans < eps) then
                !set fix flag
                osolve%kfix((i-1)*3+1)=1
                osolve%kfix((i-1)*3+3)=1

                !set velocity
                osolve%u(i)=vx
                osolve%w(i)=vz
            end if
        else
        !Use horzflow only when indenter does not cut face
            !horzflow: flow fixed to (vback, 0 0)
            if (horzflow) then
                osolve%kfix((i-1)*3+1)=1 ; osolve%u(i)=vback
                osolve%kfix((i-1)*3+3)=1 ; osolve%w(i)=0.d0
            end if
        end if
        !more constrained horizontal flow, force half of y=0 side beyond indenter
        !footprint to have vx = vback
        if (horzsides .AND. osolve%x(i) > (xbotedgey0 + 1.d0) * 0.5d0) then
            osolve%kfix((i-1)*3+1)=1 ; osolve%u(i)=vback
        end if
    end if

    !y=1 face free slip
    if (osolve%y(i)>(1.d0-eps)) then
        !no flux through face
        osolve%kfix((i-1)*3+2)=1 ; osolve%v(i)=0.d0
        
        !rotating sphere cuts y=1 face
        if (cuty1face) then
            if (disttrans < eps) then
                !set fix flag
                osolve%kfix((i-1)*3+1)=1
                osolve%kfix((i-1)*3+3)=1

                !set velocity
                osolve%u(i)=vx
                osolve%w(i)=vz
            end if
        else
        !Use horzflow only when indenter does not cut face
            !horzflow: flow fixed to (vback, 0 0)
            if (horzflow) then
                osolve%kfix((i-1)*3+1)=1 ; osolve%u(i)=vback
                osolve%kfix((i-1)*3+3)=1 ; osolve%w(i)=0.d0
            end if
        end if
        !more constrained horizontal flow, force half of y=0 side beyond indenter
        !footprint to have vx = vback
        if (horzsides .AND. osolve%x(i) > (xbotedgey1 + 1.d0) * 0.5d0) then
            osolve%kfix((i-1)*3+1)=1 ; osolve%u(i)=vback
        end if
    end if
    
    !x=0 or x=1 face
    if (osolve%x(i)<eps .OR. osolve%x(i)>1.d0-eps) then
        osolve%kfix((i-1)*3+1)=1 !x-direction fixed

        !set y and z velocity always for x=0
        !for x=1 only inside indenter or if horzflow 
        if (osolve%x(i)<eps .OR. &
             (osolve%x(i)>1.d0-eps .AND. disttrans < eps) .OR. &
             (osolve%x(i)>1.d0-eps .AND. horzflow .AND. (.NOT. cutx1face))) then
            osolve%kfix((i-1)*3+2)=1 !y-direction
            osolve%kfix((i-1)*3+3)=1 !z-direction
        end if
        
        !if properly inside the indenter, set velocity
        if (disttrans < eps) then
            osolve%u(i) = vx
            osolve%v(i) = 0.d0
            osolve%w(i) = vz
        else
            !velo above indenter is v_top or v_back for x=0, 
            !but always v_back for x=1
            if (osolve%x(i) < eps) then
                vx_above = vtop + distzoney*(vback-vtop)
            else
                vx_above = vback
            end if

            !if outside indenter
            if (disttrans > 1.d0-eps) then
                osolve%u(i) = vx_above
                osolve%v(i) = 0.d0
                osolve%w(i) = 0.d0
            !if in indenter transition zone
            else
                osolve%u(i) = vx + disttrans*(vx_above-vx)
                osolve%v(i) = 0.d0
                osolve%w(i) = vz * (1.d0-disttrans) !=vz + disttrans*(vz_above(=0)-vz)
            end if
        end if
    end if

    !z=0 face
    if (osolve%z(i)*params%vex < eps) then
        !fix all velocities and temp, set vy to 0
        osolve%kfix((i-1)*3+1)=1 
        osolve%kfix((i-1)*3+2)=1 ; osolve%v(i)=0.d0 !y-direction
        osolve%kfix((i-1)*3+3)=1
        osolve%kfixt(i)=1 
        
        !if properly within indenter
        if (disttrans < 0.d0+eps) then
            !set velocity
            osolve%u(i) = vx
            osolve%v(i) = 0.d0
            osolve%w(i) = vz
            
            if (freeindbot) then
                !unfix vz
                osolve%kfix((i-1)*3+3)=0
            end if

            !set temperature
            select case(tempcase)
                case (-2)
                    osolve%temp(i) = max(temp2, osolve%temp(i))
                case (-1)
                    continue
                case (0,1)
                    osolve%temp(i)=1.d0
                case (2)
                    osolve%temp(i) = 1.0d0+(distrot+zc)/h1*(temp2-1.d0)
                case default
                    osolve%temp(i)=1.d0
            end select
        else
            !if outside indenter
            if (disttrans > 1.d0-eps) then
                !set velocity
                osolve%u(i) = vback
                osolve%v(i) = 0.d0
                osolve%w(i) = 0.d0

                if (botoutflux) then
                    distsubzone = (1.d0 - osolve%x(i))/sublength
                    if (distsubzone > 1.d0 .OR. distsubzone < eps) then
                        distsubzone = 0.d0
                    end if
                    osolve%w(i) = distsubzone * wsub
                end if

                !set temperature
                select case(tempcase)
                    case (-2)
                        osolve%temp(i) = max(temp2, osolve%temp(i))
                    case (-1)
                        continue
                    case (0,1,2)
                        osolve%temp(i)=1.d0
                    case default
                        osolve%temp(i)=1.d0
                end select

            !if in indenter transition zone
            else
                osolve%u(i) = vx + disttrans*(vback-vx)
                osolve%v(i) = 0.d0
                if (botoutflux) then
                    distsubzone = (1.d0 - osolve%x(i))/sublength
                    distsubzone = min(1.d0, max(0.d0, distsubzone))
                    wsubend = distsubzone * wsub
                else
                    wsubend = 0.d0
                end if
                osolve%w(i) = vz + disttrans * (wsubend - vz)

                if (freeindbot) then
                    !unfix vz
                    osolve%kfix((i-1)*3+3)=0
                end if

                !set temperature
                select case(tempcase)
                    case (-2)
                        osolve%temp(i) = max(temp2, osolve%temp(i))
                    case (-1)
                        continue
                    case (0)
                        osolve%temp(i)=1.d0
                    case (1)
                        tstart = temp2
                        osolve%temp(i) = tstart + disttrans*(1.d0-tstart)
                    case (2)
                        tstart = 1.0d0+(distrot+zc)/h1*(temp2-1.d0)
                        osolve%temp(i) = tstart + disttrans*(1.d0-tstart)
                    case default
                        osolve%temp(i)=1.d0
                end select
            end if
        end if
    end if

    !z=1 face 
    if (osolve%z(i) > 1.d0-eps) then
        osolve%kfix((i-1)*3+1)=1 ; osolve%u(i)=0.d0 !x-direction
        osolve%kfix((i-1)*3+2)=1 ; osolve%v(i)=0.d0 !y-direction
        osolve%kfix((i-1)*3+3)=1 ; osolve%w(i)=0.d0 !z-direction
        osolve%kfixt(i)=1 ; osolve%temp(i)=0.d0
    end if
    
    if (.not.vo%influid(i)) then
       osolve%kfixt(i)=1
       osolve%temp(i)=0.d0
    end if

    !if (debugging) then
        !debug1(i) = osolve%kfix((i-1)*3+1)
        !debug2(i) = osolve%kfix((i-1)*3+2)
        !debug3(i) = osolve%kfix((i-1)*3+3)
        !debug4(i) = osolve%u(i)
    !end if

end do

!if (debugging) then
    !!define index helpers
    !nne = 8
    !nnn=0
    !nnne=0
    !do i=1,osolve%nnode
        !if (vo%node(i).eq.0) nnn=nnn+1
    !end do
    !do i=1,osolve%nleaves
        !if (vo%leaf(i).eq.0) nnne=nnne+1
    !end do
    !
    !!write output
    !open(unit=33,file='DEBUG/bcdef.vtk')
    !write(33,'(a)')'# vtk DataFile Version 3.0'
    !write(33,'(a)')'transition zones'
    !write(33,'(a)')'ASCII'
    !write(33,'(a)')'DATASET UNSTRUCTURED_GRID'
    !write(33,'(a7,i10,a6)')'POINTS ',nnn,' float'
    !
    !do i=1,nnn
    !write(33,'(3f16.11)')osolve%x(vo%rtf(i)),osolve%y(vo%rtf(i)),osolve%z(vo%rtf(i))*params%vex
    !enddo
    !
    !write(33,'(A6, 2I10)') 'CELLS ',nnne,(1+nne)*nnne
    !iconmin=osolve%nnode
    !iconmax=0
    !do ie=1,osolve%nleaves
       !if (vo%leaf(ie).eq.0) then
       !myicon(1:nne)=vo%ftr(osolve%icon(1:nne,ie))-1
         !do k=1,nne
         !iconmin=min(iconmin,myicon(k))
         !iconmax=max(iconmax,myicon(k))
         !enddo
       !write(33,'(9I10)')nne,myicon(1:nne)
       !endif
    !enddo
    !
    !write(33,'(A11, I10)') 'CELL_TYPES ',nnne
    !do ie=1,nnne
       !write(33,'(I2)')11 ! octree  (8 nodes)
    !end do
    !
    !write(33,'(a11,i10)')'POINT_DATA ',nnn
    !
    !write(33,'(a)')'SCALARS kfix_vx double 1'
    !write(33,'(a)')'LOOKUP_TABLE default'
    !do i=1,nnn
        !write(33,'(e13.6)') debug1(vo%rtf(i))
    !enddo
    !
    !write(33,'(a)')'SCALARS kfix_vy double 1'
    !write(33,'(a)')'LOOKUP_TABLE default'
    !do i=1,nnn
        !write(33,'(e13.6)') debug2(vo%rtf(i))
    !enddo
    !
    !write(33,'(a)')'SCALARS kfix_vz double 1'
    !write(33,'(a)')'LOOKUP_TABLE default'
    !do i=1,nnn
        !write(33,'(e13.6)') debug3(vo%rtf(i))
    !enddo
    !
    !write(33,'(a)')'SCALARS u double 1'
    !write(33,'(a)')'LOOKUP_TABLE default'
    !do i=1,nnn
        !write(33,'(e13.6)') debug4(vo%rtf(i))
    !enddo
    !
    !
    !write(33,'(a)')'VECTORS velo double' 
    !do i=1,nnn 
        !write(33,'(3e17.6)')osolve%u(vo%rtf(i)),osolve%v(vo%rtf(i)),osolve%w(vo%rtf(i))
    !enddo 
    !
    !close (33)
    !
    !deallocate(debug1)
    !deallocate(debug2)
    !deallocate(debug3)
    !deallocate(debug4)
!end if

end subroutine define_bc_rot_subduction
!------------------------------------------------------------------------------|
end module m_define_bc_rot_subduction

