module m_remove_bad
    contains

recursive subroutine remove_bad_triangles (surface, surface0, flag0, is, params, ref_count, nremoved)

use definitions
use m_toolbox

implicit none

include 'mpif.h'

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! This routine removes broken triangles, i.e. triangles that consist only of 
! two points. Those are removed without any replacement.

!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

type(sheet) :: surface, surface0
type(parameters) :: params

integer :: flag0, is, ref_count, nremoved

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|

logical :: dupl_found

integer :: nproc,iproc,ierr
integer :: i, j, k, ii, inode, counter, tr_counter, crit_counter
integer :: kp, inodep, ncritmax, nneighb
integer :: deadcnt, ndead, crittri_counter
integer :: numremtri, new_nsurface, new_nt
integer, dimension(:), allocatable :: deadlist, deadnode, critnodes
integer, dimension(:), allocatable :: remtri, new_ch
integer, dimension(:,:), allocatable :: new_icon, crittri, allneighbours

double precision, dimension(:), allocatable :: new_x,new_y,new_z
double precision, dimension(:), allocatable :: new_xn,new_yn,new_zn
double precision, dimension(:), allocatable :: new_x0,new_y0,new_z0
double precision, dimension(:), allocatable :: new_xn0,new_yn0,new_zn0

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------

call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)

if (iproc == 0) then
    write (8,'(a)') "Start of remove_bad_triangles" 
    if (params%debug > 1 .and. ref_count > 0) then
        call output_surf (surface, is, "remove_bad_triangles", params%current_step,(ref_count-1)*2+1)
    end if
end if

!create neighbour list for all nodes and check for triangles with identical points
numremtri = 0
allocate (allneighbours(-1:13,surface%nsurface))
allneighbours = 0

do i=1,surface%nt
    !for each triangle, check for identical nodes
    checktri: do j=1,3
      do k = 0,1
        !process both neighbours of node i in this triangle
        inode=surface%icon(j,i)
        kp=mod(j+k,3)+1
        inodep=surface%icon(kp,i)

        if (inode /= inodep) then
        !add to neighbour_list
            if (allneighbours(0,inode) == 0) then
                allneighbours(0,inode) = 1
                allneighbours(1,inode) = inodep
            else
                !only add if not already in neighbour list
                nneighb = allneighbours(0,inode) 
                if (count( allneighbours(1:nneighb,inode) == inodep) == 0) then 
                    allneighbours(0,inode) = nneighb+1
                    allneighbours(nneighb+1,inode) = inodep
                end if
            end if
        else
            !same node has two entries in this triangle, mark as critical
            allneighbours(-1,inode) = 1
        end if
      end do
    end do checktri
end do

!check number of neighbours for all nodes
ncritmax = 0
do i = 1,surface%nsurface
    if (surface%convexhull(i) > 0) then
        if (allneighbours(0,i) < 2) allneighbours(-1,i) = 1
        ncritmax = ncritmax + allneighbours(0,i) + 1
    else
        if (allneighbours(0,i) < 3) allneighbours(-1,i) = 1
        ncritmax = ncritmax + allneighbours(0,i) + 1
    end if
end do

!collect all critical nodes AND their neighbours
allocate(critnodes(ncritmax))
critnodes = 0
crit_counter = 0

if (iproc == 0) write (8,*) "These are the critical nodes: "
do inode=1,surface%nsurface
    if (allneighbours(-1,inode) > 0) then
        if (crit_counter == 0) then
            crit_counter = 1
            critnodes(1) = inode
            if (iproc==0) write (8,*) inode
        else if (count (critnodes(1:crit_counter) == inode) == 0) then
            crit_counter = crit_counter + 1
            critnodes(crit_counter) = inode
            if (iproc==0) write (8,*) inode
        end if
        do j=1,allneighbours(0,inode)
            inodep = allneighbours(j,inode)
            if (count (critnodes(1:crit_counter) == inodep) == 0) then
                crit_counter = crit_counter + 1
                critnodes(crit_counter) = inodep
                if (iproc==0) write (8,*) inodep
            end if
        end do
    end if
end do
deallocate (allneighbours)

if (crit_counter > 0) then
    !make a shorter list of all triangles containing critical nodes
    allocate (crittri(0:3,1:crit_counter*13))
    crittri = 0

    crittri_counter = 0
    do i = 1,surface%nt
        do j=1,3
            if (count(critnodes(1:crit_counter) == surface%icon(j,i)) > 0) then
                crittri_counter = crittri_counter+1
                crittri(0, crittri_counter) = i
                crittri(1:3, crittri_counter) = surface%icon(1:3, i)
                exit
            end if
        end do
    end do

    if (iproc == 0 .and. params%debug > 1) then
        write (8,*) "ctrittri array:"
        do i = 1, crittri_counter
            write (8,*) crittri(:, i)
        end do
    end if
else
    !use entire icon as crittri if no critical nodes were found
    allocate(crittri(0:3,surface%nt))

    do i=1,surface%nt
        crittri(0,i) = i
        crittri(1:3,i) = surface%icon(1:3, i)
    end do
    crittri_counter = surface%nt
    if (iproc == 0 .and. params%debug >= 1) then
        write (8,*) "No critical nodes found, entire icon is used as crittri array"
    end if
end if

!now check all triangles if they consist of only two nodes or are a duplicate of
!another triangle. Those triangles are skipped in the following copy section.
allocate (remtri(1:crittri_counter))
remtri = 0
numremtri = 0

check_crittri: do i=1,crittri_counter
    !check if bad triangles 
    do j=1,2
        do k = j+1,3
            if (crittri(j,i)==crittri(k,i)) then
                numremtri = numremtri+1
                remtri(numremtri)  = crittri(0,i)
                cycle check_crittri
            end if
        end do
    end do
    !if not bad triangle, check if same triangle exist in rest of the array
    checkdupl: do ii = i+1, crittri_counter
        do j=1,3
            !if no duplicate is found in any one of three comparisons, directly
            !go to next triangle
            dupl_found = .false.
            do k = 1,3
                if (crittri(j,i) == crittri(k,ii)) then
                    dupl_found = .true.
                    exit
                end if
            end do
            if (.not. dupl_found) cycle checkdupl
        end do
        ! only get here when triangle is identical
        if (iproc==0) then
            write (8,*) "Found identical triangles:"
            write (8,'(3i8)') crittri(1:3,i)
            write (8,'(3i8)') crittri(1:3,ii)
        end if

        !add both triangles to the list of triangles to be deleted
        !if triangle exist more than twice, first triangle should already be in
        !remtri, so check here.
        if (count(remtri(1:numremtri) == crittri(0,i)) == 0) then
            numremtri = numremtri+2
            remtri(numremtri-1)  = crittri(0,i)
        else
            numremtri = numremtri+1
        end if
        remtri(numremtri)  = crittri(0,ii)
        
        ! if found two identical, directly go to next triangle. Higher
        ! re-occurences will be found when i has progessed to ii
        cycle check_crittri

    end do checkdupl
end do check_crittri

if (numremtri > 0) then
    if (iproc == 0) write (8,'(a, i3, a)') "Removing the following ", numremtri, " triangles in surface%icon:"

    !remove bad triangles
    new_nt=surface%nt-numremtri
    allocate(new_icon(1:3,1:new_nt))

    tr_counter = 0
    copytri: do i=1,surface%nt
        if (count(remtri(1:numremtri) == i) == 0) then
            tr_counter = tr_counter+1
            new_icon(1:3,tr_counter) = surface%icon(1:3,i)
        else
            if (iproc==0) write (8,'(3i10)') surface%icon(1:3,i)
        end if
    end do copytri


    !copy new array to new surface%icon
    deallocate(surface%icon)

    surface%nt=new_nt
    allocate(surface%icon(1:3,1:surface%nt))

    surface%icon = new_icon
    deallocate (new_icon)
else
    if (iproc == 0) write (8,'(a, i3, a)') "No triangles found to be removed"
end if

! now go through all triangles and count the number of occurences for each node
! to identify dead nodes, i.e. nodes that now are part of no triangle.

allocate (deadnode(1:surface%nsurface))
deadnode = 0

do i = 1,surface%nt
  do j = 1,3
    k = surface%icon(j,i) 
    deadnode(k) = deadnode(k) + 1
  end do
end do

ndead = count(deadnode == 0)
if (ndead > 0) then
    allocate (deadlist(ndead))
    deadlist = 0

    deadcnt = 0
    do i = 1,surface%nsurface
        if (deadnode(i) == 0) then
            deadcnt = deadcnt +1
            deadlist(deadcnt) = i
        end if
    end do

    if (iproc == 0) then
        write (8,'(i5,a)') ndead, " nodes without triangles were found:"
        do i= 1, ndead
            write (8,*) deadlist(i)
        end do
    end if

    new_nsurface= surface%nsurface-ndead

    allocate(new_x(new_nsurface))
    allocate(new_y(new_nsurface))
    allocate(new_z(new_nsurface))
    allocate(new_xn(new_nsurface))
    allocate(new_yn(new_nsurface))
    allocate(new_zn(new_nsurface))
    allocate(new_ch(new_nsurface))

    if (flag0==1) then
      allocate(new_x0(new_nsurface))
      allocate(new_y0(new_nsurface))
      allocate(new_z0(new_nsurface))
      allocate(new_xn0(new_nsurface))
      allocate(new_yn0(new_nsurface))
      allocate(new_zn0(new_nsurface))
    end if


    !=====[update icon indices]=====

    do i=1,surface%nt
       do k=1,3
          inode = surface%icon(k,i)
          surface%icon(k,i) = inode - count(deadlist < inode)
       end do
    end do

    !=====[update coordinates]=====

    counter=0
    do i=1,surface%nsurface
       if (count(deadlist == i) == 0) then
          counter=counter+1

          new_x  (counter)=surface%x(i)
          new_y  (counter)=surface%y(i)
          new_z  (counter)=surface%z(i)
          new_xn (counter)=surface%xn(i)
          new_yn (counter)=surface%yn(i)
          new_zn (counter)=surface%zn(i)
          new_ch (counter)=surface%convexhull(i)

          if (flag0==1) then
            new_x0 (counter)=surface0%x(i)
            new_y0 (counter)=surface0%y(i)
            new_z0 (counter)=surface0%z(i)
            new_xn0(counter)=surface0%xn(i)
            new_yn0(counter)=surface0%yn(i)
            new_zn0(counter)=surface0%zn(i)
          end if
       end if
    end do

    deallocate(surface%x)
    deallocate(surface%y)
    deallocate(surface%z)
    deallocate(surface%xn)
    deallocate(surface%yn)
    deallocate(surface%zn)
    deallocate(surface%convexhull)

    if (flag0==1) then
      deallocate(surface0%x)
      deallocate(surface0%y)
      deallocate(surface0%z)
      deallocate(surface0%xn)
      deallocate(surface0%yn)
      deallocate(surface0%zn)
    end if

    surface%nsurface=new_nsurface
    allocate(surface%x(surface%nsurface))
    allocate(surface%y(surface%nsurface))
    allocate(surface%z(surface%nsurface))
    allocate(surface%xn(surface%nsurface))
    allocate(surface%yn(surface%nsurface))
    allocate(surface%zn(surface%nsurface))
    allocate(surface%convexhull(surface%nsurface))

    if (flag0==1) then
      surface0%nsurface=new_nsurface
      allocate(surface0%x(surface0%nsurface))
      allocate(surface0%y(surface0%nsurface))
      allocate(surface0%z(surface0%nsurface))
      allocate(surface0%xn(surface0%nsurface))
      allocate(surface0%yn(surface0%nsurface))
      allocate(surface0%zn(surface0%nsurface))
    end if

    surface%x=new_x
    surface%y=new_y
    surface%z=new_z
    surface%xn=new_xn
    surface%yn=new_yn
    surface%zn=new_zn
    surface%convexhull=new_ch

    if (flag0==1) then
      surface0%x=new_x0
      surface0%y=new_y0
      surface0%z=new_z0
      surface0%xn=new_xn0
      surface0%yn=new_yn0
      surface0%zn=new_zn0
    end if


    deallocate(new_x)
    deallocate(new_y)
    deallocate(new_z)
    deallocate(new_xn)
    deallocate(new_yn)
    deallocate(new_zn)
    deallocate(new_ch)

    if (flag0==1) then
      deallocate(new_x0)
      deallocate(new_y0)
      deallocate(new_z0)
      deallocate(new_xn0)
      deallocate(new_yn0)
      deallocate(new_zn0)
    end if

    deallocate (deadlist)
    nremoved = nremoved + ndead
    if (iproc==0) then
        write (*,'(i4,a)') ndead, " nodes removed in remove_bad_triangles"
        write (8,'(i4,a)') ndead, " nodes removed in remove_bad_triangles"
    end if
else
    if (iproc==0) write (8,'(a)') "No nodes without triangles were found:"
end if

deallocate (deadnode, critnodes, remtri)
deallocate (crittri) 

if (ndead==0 .AND. numremtri==0) then
    !Found no triagnles or nodes to be removed, successfully exit
    if (iproc == 0 .and. params%debug > 1) then
        call output_surf (surface, is, "remove_bad_triangles", params%current_step, (abs(ref_count)-1)*2+2) 
    end if
else
    !call again to check for newly created  bad nodes and triangles.
    call remove_bad_triangles (surface, surface0, flag0, is, params, -1*abs(ref_count), nremoved)
end if

end subroutine remove_bad_triangles 

!------------------------------------------------------------------------------|

subroutine remove_bad_points (point_list,npoint,surface,flag0,surface0)

use definitions
use m_toolbox

implicit none

include 'mpif.h'

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! This routine removes bad points, i.e. points that are part of triangles with
! only two edges. Those nodes need special treatment.

!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

integer npoint,flag0
integer, dimension(npoint) :: point_list
type(sheet) :: surface,surface0

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|

logical :: foundremnode

integer :: nproc,iproc,ierr
integer :: i,j,k,counter,tr_counter
integer :: irem, numrem, numremtri, new_nsurface, new_nt
integer, dimension(:,:), allocatable :: new_icon
integer, dimension(:), allocatable :: remove_list, new_ch

double precision, dimension(:), allocatable :: new_x,new_y,new_z
double precision, dimension(:), allocatable :: new_xn,new_yn,new_zn
double precision, dimension(:), allocatable :: new_x0,new_y0,new_z0
double precision, dimension(:), allocatable :: new_xn0,new_yn0,new_zn0

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------

call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)


if (iproc==0) write (*,*) 'WARNING: remove_bad_points called'

!===============================================================================
!=====[Check for points on hull that would be removed]==========================
!===============================================================================

numrem = 0
do i=1,npoint
    !do not remove corner points or same point twice
    if ( (.NOT. btest(surface%convexhull(point_list(i)), 0)) .AND. &
         (count(point_list(1:i) - point_list(i) == 0) == 1)) then
        numrem = numrem + 1
    end if
end do

if (numrem==npoint) then
    !just copy point_list to remove_list
    allocate (remove_list(numrem))
    remove_list = point_list
    irem = numrem
elseif (numrem > 0) then
    !Disable removal of corner nodes
    if (iproc==0) write (*,*) 'WARNING: ',npoint-numrem,' corner nodes of the', &
        ' convex hull were marked for removal in remove_bad_nodes'
    if (iproc==0) write (8,*) 'WARNING: ',npoint-numrem,' corner nodes of the', &
        'convex hull were marked for removal in remove_bad_nodes'

    allocate (remove_list(numrem))
    irem = 0
    do i = 1,npoint
        if ( (.NOT. btest(surface%convexhull(point_list(i)), 0)) .AND. &
             (count(point_list(1:i) - point_list(i) == 0) == 1)) then
            irem = irem+1
            remove_list(irem) = point_list(i)
        else
            if (iproc==0) then
                if ( (.NOT. btest(surface%convexhull(point_list(i)), 0))) then
                    write(8,*) 'Not removing corner point ',point_list(i),' from convex hull'
                    write(8,'(a,3f10.6)') 'position: ', surface%x(point_list(i)), &
                        surface%y(point_list(i)), surface%z(point_list(i))
                else if (count(point_list(1:i) - point_list(i) == 0) == 1) then
                    write(8,*) 'Point ',point_list(i),' twice in remove_list'
                end if
            end if
        end if
    end do
else
    if (iproc==0) then
        write (*,*) 'WARNING: Only corner nodes marked for removal'
        write (*,*) 'Skipping remove_bad_nodes'
        write (8,*) 'WARNING: Only corner nodes marked for removal'
        write (8,*) 'Skipping remove_bad_nodes'
    end if

    return
end if
numrem = irem

if (iproc==0) then
    do i = 1,numrem
        k = remove_list(i)
        write (8, '(i6,a,3f10.6)') k, ': ', surface%x(k), surface%y(k), &
            surface%z(k)
    end do
end if


!Count triangles to be removed
numremtri = 0
do i=1,surface%nt
    !for each triangle, check for nodes of remove_list
    thistri: do j=1,3
        if (count(surface%icon(j,i)==remove_list) > 0) then
            numremtri = numremtri + 1
            exit thistri
        end if
    end do thistri
end do

!-------------------------------------------------------------------------------
! carry out point removal
!-------------------------------------------------------------------------------

new_nsurface=surface%nsurface-numrem
new_nt=surface%nt-numremtri

allocate(new_icon(3,new_nt))
allocate(new_x(new_nsurface))
allocate(new_y(new_nsurface))
allocate(new_z(new_nsurface))
allocate(new_xn(new_nsurface))
allocate(new_yn(new_nsurface))
allocate(new_zn(new_nsurface))
allocate(new_ch(new_nsurface))

if (flag0==1) then
allocate(new_x0(new_nsurface))
allocate(new_y0(new_nsurface))
allocate(new_z0(new_nsurface))
allocate(new_xn0(new_nsurface))
allocate(new_yn0(new_nsurface))
allocate(new_zn0(new_nsurface))
end if

!=====[update icon]=====

tr_counter=0
do i=1,surface%nt
    !for each triangle, check for nodes of remove_list
    foundremnode = .false.
    do j=1,3
        if (count(surface%icon(j,i)==remove_list) > 0) then
            foundremnode=.true.
            exit
        end if
    end do

    !keep triangle only if it has none of remove_points, otherwise discard
    if (.NOT. foundremnode) then
        tr_counter = tr_counter + 1
        do j=1,3
            k = surface%icon(j,i)
            !shift node ID down by number of remove points smaller than this
            !point's id
            new_icon(j,tr_counter) = k - count(remove_list < k)
        end do
    end if
end do

!=====[update node lists]=====
counter=0
do i=1,surface%nsurface
   if (count(i==remove_list) == 0) then
      counter=counter+1

      new_x  (counter)=surface%x(i)
      new_y  (counter)=surface%y(i)
      new_z  (counter)=surface%z(i)
      new_xn (counter)=surface%xn(i)
      new_yn (counter)=surface%yn(i)
      new_zn (counter)=surface%zn(i)
      new_ch (counter)=surface%convexhull(i)

      if (flag0==1) then
      new_x0 (counter)=surface0%x(i)
      new_y0 (counter)=surface0%y(i)
      new_z0 (counter)=surface0%z(i)
      new_xn0(counter)=surface0%xn(i)
      new_yn0(counter)=surface0%yn(i)
      new_zn0(counter)=surface0%zn(i)
      end if

   end if
end do

deallocate(surface%x)
deallocate(surface%y)
deallocate(surface%z)
deallocate(surface%xn)
deallocate(surface%yn)
deallocate(surface%zn)
deallocate(surface%convexhull)
deallocate(surface%icon)

if (flag0==1) then
deallocate(surface0%x)
deallocate(surface0%y)
deallocate(surface0%z)
deallocate(surface0%xn)
deallocate(surface0%yn)
deallocate(surface0%zn)
end if

surface%nt=new_nt
surface%nsurface=new_nsurface

allocate(surface%x(surface%nsurface))
allocate(surface%y(surface%nsurface))
allocate(surface%z(surface%nsurface))
allocate(surface%xn(surface%nsurface))
allocate(surface%yn(surface%nsurface))
allocate(surface%zn(surface%nsurface))
allocate(surface%convexhull(surface%nsurface))
allocate(surface%icon(3,surface%nt))

if (flag0==1) then
surface0%nsurface=new_nsurface
allocate(surface0%x(surface0%nsurface))
allocate(surface0%y(surface0%nsurface))
allocate(surface0%z(surface0%nsurface))
allocate(surface0%xn(surface0%nsurface))
allocate(surface0%yn(surface0%nsurface))
allocate(surface0%zn(surface0%nsurface))
end if

surface%x=new_x
surface%y=new_y
surface%z=new_z
surface%xn=new_xn
surface%yn=new_yn
surface%zn=new_zn
surface%convexhull=new_ch
surface%icon=new_icon

if (flag0==1) then
surface0%x=new_x0
surface0%y=new_y0
surface0%z=new_z0
surface0%xn=new_xn0
surface0%yn=new_yn0
surface0%zn=new_zn0
end if

deallocate(new_x)
deallocate(new_y)
deallocate(new_z)
deallocate(new_icon)
deallocate(new_xn)
deallocate(new_yn)
deallocate(new_zn)
deallocate(new_ch)

if (flag0==1) then
deallocate(new_x0)
deallocate(new_y0)
deallocate(new_z0)
deallocate(new_xn0)
deallocate(new_yn0)
deallocate(new_zn0)
end if

if (iproc == 0) write (8,*) '------------------------------------------------------------------------'
deallocate (remove_list)

end subroutine remove_bad_points

!------------------------------------------------------------------------------|

end module m_remove_bad
